-- "Set define off" turns off substitution variables
Set define off;
--
-- 1. RHN_UG_DEL_TRIG  (Trigger)
--
 DROP TRIGGER RHN_UG_DEL_TRIG;

--
-- 2. XXRH_CONTACT_DEL_CHECK_NUM  (Function)
--
-- DROP TRIGGER RHN_UG_DEL_TRIG;
 DROP FUNCTION XXRH_CONTACT_DEL_CHECK_NUM;

--
-- 3. XXRH_CONTACT_DEL_CHECK  (Function)
--
-- DROP TRIGGER RHN_UG_DEL_TRIG;
-- DROP FUNCTION XXRH_CONTACT_DEL_CHECK_NUM;
 DROP FUNCTION XXRH_CONTACT_DEL_CHECK;

--
-- 4. IS_SATELLITE  (Function)
--
-- DROP TRIGGER RHN_UG_DEL_TRIG;
-- DROP FUNCTION XXRH_CONTACT_DEL_CHECK_NUM;
-- DROP FUNCTION XXRH_CONTACT_DEL_CHECK;
 DROP FUNCTION IS_SATELLITE;

--
-- 5. RHN_PRODUCTNAME_ID_SEQ  (Sequence)
--
--
-- RHN_PRODUCTNAME_ID_SEQ  (Sequence)
--
CREATE SEQUENCE RHN_PRODUCTNAME_ID_SEQ
  START WITH 101
  MAXVALUE 1E27
  MINVALUE 1
  NOCYCLE
  CACHE 20
  NOORDER;

--
-- 6. RHNINDEXERWORK  (Table)
--
--
-- RHNINDEXERWORK  (Table)
--
CREATE TABLE RHNINDEXERWORK
(
  OBJECT_TYPE  VARCHAR2(40 BYTE)                NOT NULL,
  LAST_ID      NUMBER                           NOT NULL
)
LOGGING
NOCACHE
NOPARALLEL;

--
-- 7. RHNPRODUCTNAME  (Table)
--
--
-- RHNPRODUCTNAME  (Table)
--
CREATE TABLE RHNPRODUCTNAME
(
  ID        NUMBER CONSTRAINT RHN_PRODUCTNAME_ID_NN NOT NULL,
  LABEL     VARCHAR2(128 BYTE) CONSTRAINT RHN_PRODUCTNAME_LBL_NN NOT NULL,
  NAME      VARCHAR2(128 BYTE) CONSTRAINT RHN_PRODUCTNAME_NAME_NN NOT NULL,
  CREATED   DATE                                DEFAULT (sysdate) CONSTRAINT PRODUCT_NAME_CREATED_NN NOT NULL,
  MODIFIED  DATE                                DEFAULT (sysdate) CONSTRAINT PRODUCT_NAME_MODIFIED_NN NOT NULL
)
LOGGING
NOCACHE
NOPARALLEL;

--
-- 8. RHNRELEASECHANNELMAP  (Table)
--
--
-- RHNRELEASECHANNELMAP  (Table)
--
CREATE TABLE RHNRELEASECHANNELMAP
(
  PRODUCT          VARCHAR2(64 BYTE) CONSTRAINT RHN_RCM_PRODCT_NN NOT NULL,
  VERSION          VARCHAR2(64 BYTE) CONSTRAINT RHN_RCM_VERSN_NN NOT NULL,
  RELEASE          VARCHAR2(64 BYTE) CONSTRAINT RHN_RCM_RELSE_NN NOT NULL,
  CHANNEL_ARCH_ID  NUMBER CONSTRAINT RHN_RCM_CAID_NN NOT NULL,
  CHANNEL_ID       NUMBER CONSTRAINT RHN_RCM_CID_NN NOT NULL,
  IS_DEFAULT       CHAR(1 BYTE) CONSTRAINT RHN_RCM_DEFAULT_NN NOT NULL
)
LOGGING
NOCACHE
NOPARALLEL;

--
-- 9. RHN_PRODUCTNAME_ID_PK  (Index)
--
--
-- RHN_PRODUCTNAME_ID_PK  (Index)
--
--  Dependencies:
--   RHNPRODUCTNAME (Table)
--
CREATE UNIQUE INDEX RHN_PRODUCTNAME_ID_PK ON RHNPRODUCTNAME
(ID)
LOGGING
NOPARALLEL;

--
-- 10. RHN_PRODUCTNAME_LABEL_UQ  (Index)
--
--
-- RHN_PRODUCTNAME_LABEL_UQ  (Index)
--
--  Dependencies:
--   RHNPRODUCTNAME (Table)
--
CREATE UNIQUE INDEX RHN_PRODUCTNAME_LABEL_UQ ON RHNPRODUCTNAME
(LABEL)
LOGGING
NOPARALLEL;

--
-- 11. RHN_PRODUCTNAME_NAME_UQ  (Index)
--
--
-- RHN_PRODUCTNAME_NAME_UQ  (Index)
--
--  Dependencies:
--   RHNPRODUCTNAME (Table)
--
CREATE UNIQUE INDEX RHN_PRODUCTNAME_NAME_UQ ON RHNPRODUCTNAME
(NAME)
LOGGING
NOPARALLEL;

--
-- 12. RHN_RCM_PROD_VER_REL_CAID_IDX  (Index)
--
--
-- RHN_RCM_PROD_VER_REL_CAID_IDX  (Index)
--
--  Dependencies:
--   RHNRELEASECHANNELMAP (Table)
--
CREATE INDEX RHN_RCM_PROD_VER_REL_CAID_IDX ON RHNRELEASECHANNELMAP
(PRODUCT, VERSION, RELEASE, CHANNEL_ARCH_ID)
LOGGING
NOPARALLEL;

--
-- 13. RHN_VIS_LBL_ID_IDX  (Index)
--
--
-- RHN_VIS_LBL_ID_IDX  (Index)
--
--  Dependencies:
--   RHNVIRTUALINSTANCESTATE (Table)
--
CREATE UNIQUE INDEX RHN_VIS_LBL_ID_IDX ON RHNVIRTUALINSTANCESTATE
(LABEL, ID)
LOGGING
NOPARALLEL;

--
-- 14. RHN_VIT_LBL_ID_UQ  (Index)
--
--
-- RHN_VIT_LBL_ID_UQ  (Index)
--
--  Dependencies:
--   RHNVIRTUALINSTANCETYPE (Table)
--
CREATE UNIQUE INDEX RHN_VIT_LBL_ID_UQ ON RHNVIRTUALINSTANCETYPE
(LABEL, ID)
LOGGING
NOPARALLEL;

--
-- 15. WEB_CUSTOMER_NAME_UQ_IDX  (Index)
--
--
-- WEB_CUSTOMER_NAME_UQ_IDX  (Index)
--
--  Dependencies:
--   WEB_CUSTOMER (Table)
--
CREATE UNIQUE INDEX WEB_CUSTOMER_NAME_UQ_IDX ON WEB_CUSTOMER
(NAME)
LOGGING
NOPARALLEL;

--
-- 16. RHN_CHANNEL  (Package Body)
--
--
-- RHN_CHANNEL  (Package Body)
--
CREATE OR REPLACE PACKAGE BODY rhn_channel
IS
	body_version varchar2(100) := '$Id: rhn_channel.pkb 131724 2007-09-14 20:47:46Z jslagle $';
	cursor	base_channel_cursor(
		release_in in varchar2,
		server_arch_id_in in number,
		org_id_in in number
	) return rhnChannel%ROWTYPE is
		select distinct c.*
		from	rhnDistChannelMap			dcm,
				rhnServerChannelArchCompat	scac,
				rhnChannel					c,
				rhnChannelPermissions		cp
		where	cp.org_id = org_id_in
			and cp.channel_id = c.id
			and c.parent_channel is null
			and c.id = dcm.channel_id
			and c.channel_arch_id = dcm.channel_arch_id
			and dcm.release = release_in
			and scac.server_arch_id = server_arch_id_in
			and scac.channel_arch_id = c.channel_arch_id;
    FUNCTION get_license_path(channel_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
	license_val VARCHAR2(1000);
    BEGIN
	SELECT CFL.license_path INTO license_val
	  FROM rhnChannelFamilyLicense CFL, rhnChannelFamilyMembers CFM
	 WHERE CFM.channel_id = channel_id_in
	   AND CFM.channel_family_id = CFL.channel_family_id;
	RETURN license_val;
    EXCEPTION
	WHEN NO_DATA_FOUND
	    THEN
	    RETURN NULL;
    END get_license_path;
    PROCEDURE license_consent(channel_id_in IN NUMBER, user_id_in IN NUMBER, server_id_in IN NUMBER)
    IS
	channel_family_id_val NUMBER;
    BEGIN
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	IF rhn_channel.get_license_path(channel_id_in) IS NULL
	THEN
	    rhn_exception.raise_exception('channel_consent_no_license');
	END IF;
	INSERT INTO rhnChannelFamilyLicenseConsent (channel_family_id, user_id, server_id)
	VALUES (channel_family_id_val, user_id_in, server_id_in);
    END license_consent;
    PROCEDURE subscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, user_id_in in number := null)
    IS
        channel_parent_val      rhnChannel.parent_channel%TYPE;
        parent_subscribed       BOOLEAN;
        server_has_base_chan    BOOLEAN;
	server_already_in_chan  BOOLEAN;
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	consenting_user         NUMBER;
	allowed			number := 0;
    current_members_val     number;
    BEGIN
	if user_id_in is not null then
		allowed := rhn_channel.user_role_check(channel_id_in, user_id_in, 'subscribe');
	else
		allowed := 1;
	end if;
	if allowed = 0 then
		rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
        SELECT parent_channel INTO channel_parent_val FROM rhnChannel WHERE id = channel_id_in;
        IF channel_parent_val IS NOT NULL
        THEN
            parent_subscribed := FALSE;
            FOR check_subscription IN check_server_subscription(server_id_in, channel_parent_val)
            LOOP
                parent_subscribed := TRUE;
            END LOOP check_subscription;
            IF NOT parent_subscribed
            THEN
		RETURN;
            END IF;
        ELSE
            server_has_base_chan := FALSE;
            FOR base IN server_base_subscriptions(server_id_in)
            LOOP
                server_has_base_chan := TRUE;
            END LOOP base;
            IF server_has_base_chan
            THEN
                rhn_exception.raise_exception('channel_server_one_base');
            END IF;
        END IF;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF server_already_in_chan
        THEN
	    RETURN;
        END IF;
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_subscribe_no_family');
	END IF;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
    select current_members
    into current_members_val
    from rhnPrivateChannelFamily
    where org_id = server_org_id_val and channel_family_id = channel_family_id_val
    for update of current_members;
	available_subscriptions := rhn_channel.available_family_subscriptions(channel_family_id_val, server_org_id_val);
	IF available_subscriptions IS NULL OR
       available_subscriptions > 0 or
       can_server_consume_virt_channl(server_id_in, channel_family_id_val) = 1
	THEN
	    IF rhn_channel.get_license_path(channel_id_in) IS NOT NULL
	    THEN
		BEGIN
		SELECT user_id INTO consenting_user
		  FROM rhnChannelFamilyLicenseConsent
		 WHERE channel_family_id = channel_family_id_val
		   AND server_id = server_id_in;
		EXCEPTION
		    WHEN NO_DATA_FOUND
			THEN
			    rhn_exception.raise_exception('channel_subscribe_no_consent');
		END;
	    END IF;
	    insert into rhnServerHistory (id,server_id,summary,details) (
		select	rhn_event_id_seq.nextval,
			server_id_in,
			'subscribed to channel ' || SUBSTR(c.label, 0, 106),
			c.label
		from	rhnChannel c
		where	c.id = channel_id_in
	    );
	    UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
            INSERT INTO rhnServerChannel (server_id, channel_id) VALUES (server_id_in, channel_id_in);
	    rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
	    queue_server(server_id_in, immediate_in);
	ELSE
	    rhn_exception.raise_exception('channel_family_no_subscriptions');
	END IF;
    END subscribe_server;
    function can_server_consume_virt_channl(
        server_id_in in number,
        family_id_in in number )
    return number
    is
        cursor server_virt_families is
            select vi.virtual_system_id, cfvsl.channel_family_id
            from
                rhnChannelFamilyVirtSubLevel cfvsl,
                rhnSGTypeVirtSubLevel sgtvsl,
                rhnVirtualInstance vi
            where
                vi.virtual_system_id = server_id_in
                and sgtvsl.virt_sub_level_id = cfvsl.virt_sub_level_id
                and cfvsl.channel_family_id = family_id_in
                and exists (
                    select 1
                    from rhnServerEntitlementView sev
                    where vi.host_system_id = sev.server_id
                    and sev.server_group_type_id = sgtvsl.server_group_type_id );
    begin
        for server_virt_family in server_virt_families loop
            return 1;
        end loop;
        return 0;
    end;
    PROCEDURE bulk_subscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
        END LOOP server;
    END bulk_subscribe_server;
    PROCEDURE bulk_server_base_change(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
	    IF rhn_server.can_change_base_channel(server.element) = 1
	    THEN
                rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
                rhn_channel.subscribe_server(server.element, channel_id_in, 0, set_uid_in);
            END IF;
        END LOOP server;
    END bulk_server_base_change;
    procedure bulk_server_basechange_from(
        set_label_in in varchar2,
        set_uid_in in number,
        old_channel_id_in in number,
        new_channel_id_in in number
    ) is
    cursor servers is
        select  sc.server_id id
        from    rhnChannel nc,
                rhnServerChannelArchCompat scac,
                rhnServer s,
                rhnChannel oc,
                rhnServerChannel sc,
                rhnSet st
        where   1=1
            and st.label = set_label_in
            and st.user_id = set_uid_in
            and st.element = sc.server_id
            and sc.channel_id = old_channel_id_in
            and sc.channel_id = oc.id
            and oc.parent_channel is null
            and nc.id = new_channel_id_in
            and nc.parent_channel is null
            and sc.server_id = s.id
            and s.server_arch_id = scac.server_arch_id
            and scac.channel_arch_id = nc.channel_arch_id;
    begin
        for s in servers loop
            insert into rhnSet (
                    user_id, label, element
                ) values (
                    set_uid_in,
                    set_label_in || 'basechange',
                    s.id
                );
        end loop channel;
        bulk_server_base_change(new_channel_id_in,
                                set_label_in || 'basechange',
                                set_uid_in);
        delete from rhnSet
            where   label = set_label_in||'basechange'
                and user_id = set_uid_in;
    end bulk_server_basechange_from;
    procedure bulk_guess_server_base(
	set_label_in in varchar2,
	set_uid_in in number
    ) is
	channel_id number;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    begin
		if rhn_server.can_change_base_channel(server.element) = 1
		then
	            channel_id := guess_server_base(TO_NUMBER(server.element));
		    rhn_channel.clear_subscriptions(TO_NUMBER(server.element));
		    rhn_channel.subscribe_server(TO_NUMBER(server.element), channel_id, 0, set_uid_in);
		end if;
	    exception when others then
		null;
	    end;
	end loop server;
    end;
    function guess_server_base(
	server_id_in in number
    ) RETURN number is
	cursor server_cursor is
	    select s.server_arch_id, s.release, s.org_id
	      from rhnServer s
	     where s.id = server_id_in;
    begin
	for s in server_cursor loop
	    for channel in base_channel_cursor(s.release,
		s.server_arch_id, s.org_id)
	    loop
		return channel.id;
	    end loop base_channel_cursor;
        end loop server_cursor;
        return null;
    end;
    function normalize_server_arch(server_arch_in in varchar2)
    return varchar2
    is
	suffix VARCHAR2(128) := '-redhat-linux';
	suffix_len NUMBER := length(suffix);
    begin
	if server_arch_in is NULL then
	    return NULL;
	end if;
        if instr(server_arch_in, '-') > 0
	then
	    return server_arch_in;
	end if;
	return server_arch_in || suffix;
    end normalize_server_arch;
    function base_channel_for_release_arch(
	release_in in varchar2,
	server_arch_in in varchar2,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	server_arch varchar2(256) := normalize_server_arch(server_arch_in);
	server_arch_id number;
    begin
	begin
	    select id
	      into server_arch_id
	      from rhnServerArch
	     where label = server_arch;
	exception
	    when no_data_found then
		rhn_exception.raise_exception('server_arch_not_found');
	end;
	return base_channel_rel_archid(release_in, server_arch_id,
	    org_id_in, user_id_in);
    end base_channel_for_release_arch;
    function base_channel_rel_archid(
	release_in in varchar2,
	server_arch_id_in in number,
	org_id_in in number := -1,
	user_id_in in number := null
    ) return number is
	denied_channel_id number := null;
	valid_org_id number := org_id_in;
	valid_user_id number := user_id_in;
	channel_subscribable number;
    begin
	if org_id_in = -1 and user_id_in is not null then
	    begin
		select org_id
		  into valid_org_id
		  from web_contact
		 where id = user_id_in;
	    exception
		when no_data_found then
		    valid_user_id := null;
		    valid_org_id := -1;
	    end;
	end if;
	for c in base_channel_cursor(release_in, server_arch_id_in, valid_org_id)
	loop
	    if valid_user_id is null then
		return c.id;
	    end if;
	    select loose_user_role_check(c.id, user_id_in, 'subscribe')
	      into channel_subscribable
	      from dual;
	    if channel_subscribable = 1 then
		return c.id;
	    end if;
	    denied_channel_id := c.id;
	end loop base_channel_fetch;
	if denied_channel_id is not null then
	    rhn_exception.raise_exception('no_subscribe_permissions');
	end if;
	return NULL;
    end base_channel_rel_archid;
    procedure bulk_guess_server_base_from(
	set_label_in in varchar2,
	set_uid_in in number,
	channel_id_in in number
    ) is
	cursor channels(server_id_in in number) is
	    select	rsc.channel_id
	    from	rhnServerChannel rsc,
			rhnChannel rc
	    where	server_id_in = rsc.server_id
			and rsc.channel_id = rc.id
			and rc.parent_channel is null;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    for channel in channels(server.element)
	    loop
		if channel.channel_id = channel_id_in
		then
		    insert into rhnSet (user_id, label, element) values (set_uid_in, set_label_in || 'baseguess', server.element);
		end if;
	    end loop channel;
	end loop server;
	bulk_guess_server_base(set_label_in||'baseguess',set_uid_in);
	delete from rhnSet where label = set_label_in||'baseguess' and user_id = set_uid_in;
    end;
    PROCEDURE clear_subscriptions(server_id_in IN NUMBER, deleting_server IN NUMBER := 0 )
    IS
	cursor server_channels(server_id_in in number) is
		select	s.org_id, sc.channel_id, cfm.channel_family_id
		from	rhnServer s,
			rhnServerChannel sc,
			rhnChannelFamilyMembers cfm
		where	s.id = server_id_in
			and s.id = sc.server_id
			and sc.channel_id = cfm.channel_id;
    BEGIN
	for channel in server_channels(server_id_in)
	loop
		unsubscribe_server(server_id_in, channel.channel_id, 1, 1, deleting_server);
		rhn_channel.update_family_counts(channel.channel_family_id, channel.org_id);
	end loop channel;
    END clear_subscriptions;
    PROCEDURE unsubscribe_server(server_id_in IN NUMBER, channel_id_in NUMBER, immediate_in NUMBER := 1, unsubscribe_children_in number := 0,
                                 deleting_server IN NUMBER := 0 )
    IS
	channel_family_id_val   NUMBER;
	server_org_id_val       NUMBER;
	available_subscriptions NUMBER;
	server_already_in_chan  BOOLEAN;
	cursor	channel_family_is_proxy(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-proxy';
	cursor	channel_family_is_satellite(channel_family_id_in in number) is
		select	1
		from	rhnChannelFamily
		where	id = channel_family_id_in
		    and label = 'rhn-satellite';
	cursor local_chk_server_parent_memb (
			server_id_in number,
			channel_id_in number ) is
		select	c.id
		from	rhnChannel			c,
				rhnServerChannel	sc
		where	1=1
			and c.parent_channel = channel_id_in
			and c.id = sc.channel_id
			and sc.server_id = server_id_in;
    BEGIN
        FOR child IN local_chk_server_parent_memb(server_id_in, channel_id_in)
        LOOP
	    if unsubscribe_children_in = 1 then
		unsubscribe_server(server_id_in => server_id_in,
								channel_id_in => child.id,
								immediate_in => immediate_in,
								unsubscribe_children_in => unsubscribe_children_in,
                        deleting_server => deleting_server);
	    else
		rhn_exception.raise_exception('channel_unsubscribe_child_exists');
	    end if;
        END LOOP child;
	server_already_in_chan := FALSE;
        FOR check_subscription IN check_server_subscription(server_id_in, channel_id_in)
        LOOP
            server_already_in_chan := TRUE;
        END LOOP check_subscription;
        IF NOT server_already_in_chan
        THEN
	    RETURN;
        END IF;
   if deleting_server = 0 then
      insert into rhnServerHistory (id,server_id,summary,details) (
          select  rhn_event_id_seq.nextval,
                server_id_in,
             'unsubscribed from channel ' || SUBSTR(c.label, 0, 106),
             c.label
          from    rhnChannel c
          where   c.id = channel_id_in
      );
	UPDATE rhnServer SET channels_changed = sysdate WHERE id = server_id_in;
   end if;
   DELETE FROM rhnServerChannel WHERE server_id = server_id_in AND channel_id = channel_id_in;
   if deleting_server = 0 then
	queue_server(server_id_in, immediate_in);
   end if;
	channel_family_id_val := rhn_channel.family_for_channel(channel_id_in);
	IF channel_family_id_val IS NULL
	THEN
	    rhn_exception.raise_exception('channel_unsubscribe_no_family');
	END IF;
	for ignore in channel_family_is_satellite(channel_family_id_val) loop
		delete from rhnSatelliteInfo where server_id = server_id_in;
		delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	end loop;
	for ignore in channel_family_is_proxy(channel_family_id_val) loop
		delete from rhnProxyInfo where server_id = server_id_in;
	end loop;
	DELETE FROM rhnChannelFamilyLicenseConsent
	 WHERE channel_family_id = channel_family_id_val
	   AND server_id = server_id_in;
	SELECT org_id INTO server_org_id_val
	  FROM rhnServer
	 WHERE id = server_id_in;
	rhn_channel.update_family_counts(channel_family_id_val, server_org_id_val);
    END unsubscribe_server;
    PROCEDURE bulk_unsubscribe_server(channel_id_in IN NUMBER, set_label_in IN VARCHAR2, set_uid_in IN NUMBER)
    IS
    BEGIN
        FOR server IN rhn_set.set_iterator(set_label_in, set_uid_in)
        LOOP
            rhn_channel.unsubscribe_server(server.element, channel_id_in, 0);
        END LOOP server;
    END bulk_unsubscribe_server;
    FUNCTION family_for_channel(channel_id_in IN NUMBER)
    RETURN NUMBER
    IS
	channel_family_id_val NUMBER;
    BEGIN
	SELECT channel_family_id INTO channel_family_id_val
	  FROM rhnChannelFamilyMembers
         WHERE channel_id = channel_id_in;
	RETURN channel_family_id_val;
    EXCEPTION
        WHEN NO_DATA_FOUND
	THEN
	    RETURN NULL;
    END family_for_channel;
    FUNCTION available_family_subscriptions(channel_family_id_in IN NUMBER, org_id_in IN NUMBER)
    RETURN NUMBER
    IS
	cfp channel_family_perm_cursor%ROWTYPE;
	current_members_val NUMBER;
	max_members_val     NUMBER;
	found               NUMBER;
    BEGIN
	IF NOT channel_family_perm_cursor%ISOPEN
	THEN
	    OPEN channel_family_perm_cursor(channel_family_id_in, org_id_in);
	END IF;
	FETCH channel_family_perm_cursor INTO cfp;
	WHILE channel_family_perm_cursor%FOUND
	LOOP
	    found := 1;
	    current_members_val := cfp.current_members;
	    max_members_val := cfp.max_members;
	    FETCH channel_family_perm_cursor INTO cfp;
	END LOOP;
	IF channel_family_perm_cursor%ISOPEN
	THEN
	    CLOSE channel_family_perm_cursor;
	END IF;
	IF found IS NULL
	THEN
	    RETURN 0;
	END IF;
	IF max_members_val IS NULL
	THEN
	    RETURN NULL;
	END IF;
	RETURN max_members_val - current_members_val;
    END available_family_subscriptions;
    function channel_family_current_members(channel_family_id_in IN NUMBER,
                                            org_id_in IN NUMBER)
    return number
    is
        current_members_count number := 0;
    begin
        select	count(sc.server_id)
        into    current_members_count
        from	rhnChannelFamilyMembers cfm,
                rhnServerChannel sc,
                rhnServer s
        where	s.org_id = org_id_in
            and s.id = sc.server_id
            and cfm.channel_family_id = channel_family_id_in
            and cfm.channel_id = sc.channel_id
            and exists (
                select 1
                from rhnChannelFamilyServerPhysical cfsp
                where cfsp.CHANNEL_FAMILY_ID = channel_family_id_in
                    and cfsp.server_id = s.id
                );
        return current_members_count;
    end;
    PROCEDURE update_family_counts(channel_family_id_in IN NUMBER,
                                   org_id_in IN NUMBER)
    IS
    BEGIN
		update rhnPrivateChannelFamily
		set current_members = (
                channel_family_current_members(channel_family_id_in, org_id_in)
		)
			where org_id = org_id_in
				and channel_family_id = channel_family_id_in;
    END update_family_counts;
    FUNCTION available_chan_subscriptions(channel_id_in IN NUMBER,
                                          org_id_in IN NUMBER)
    RETURN NUMBER
    IS
	    channel_family_id_val NUMBER;
    BEGIN
	SELECT channel_family_id INTO channel_family_id_val
	    FROM rhnChannelFamilyMembers
	    WHERE channel_id = channel_id_in;
	    RETURN rhn_channel.available_family_subscriptions(
                           channel_family_id_val, org_id_in);
    END available_chan_subscriptions;
    procedure entitle_customer(customer_id_in in number,
                               channel_family_id_in in number,
                               quantity_in in number)
    is
		cursor permissions is
			select	1
			from	rhnPrivateChannelFamily pcf
			where	pcf.org_id = customer_id_in
				and	pcf.channel_family_id = channel_family_id_in;
    begin
		for perm in permissions loop
			set_family_maxmembers(
				customer_id_in,
				channel_family_id_in,
				quantity_in
			);
			rhn_channel.update_family_counts(
				channel_family_id_in,
				customer_id_in
			);
			return;
		end loop;
		insert into rhnPrivateChannelFamily pcf (
				channel_family_id, org_id, max_members, current_members
			) values (
				channel_family_id_in, customer_id_in, quantity_in, 0
			);
    end;
    procedure set_family_maxmembers(customer_id_in in number,
                                    channel_family_id_in in number,
                                    quantity_in in number)
    is
        cursor servers is
            select  server_id from (
            select	rownum row_number, server_id, modified from (
                select  rcfsp.server_id,
                        rcfsp.modified
                from    rhnChannelFamilyServerPhysical rcfsp
                where   rcfsp.customer_id = customer_id_in
                    and rcfsp.channel_family_id = channel_family_id_in
                order by modified
            )
            where rownum > quantity_in
            );
    begin
        for server in servers loop
            rhn_channel.unsubscribe_server_from_family(server.server_id,
                                                       channel_family_id_in);
        end loop;
        update	rhnPrivateChannelFamily pcf
        set	pcf.max_members = quantity_in
        where	pcf.org_id = customer_id_in
            and pcf.channel_family_id = channel_family_id_in;
    end;
    procedure unsubscribe_server_from_family(server_id_in in number,
                                             channel_family_id_in in number)
    is
    begin
        delete
        from	rhnServerChannel rsc
        where	rsc.server_id = server_id_in
            and channel_id in (
                select	rcfm.channel_id
                from	rhnChannelFamilyMembers rcfm
                where	rcfm.channel_family_id = channel_family_id_in);
    end;
    function get_org_id(channel_id_in in number)
    return number
    is
	org_id_out number;
    begin
	select org_id into org_id_out
	    from rhnChannel
	    where id = channel_id_in;
	    return org_id_out;
    end get_org_id;
    function get_cfam_org_access(cfam_id_in in number, org_id_in in number)
    return number
    is
	cursor	families is
			select	1
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = org_id_in;
    begin
		for family in families loop
		return 1;
		end loop;
		return 0;
    end;
    function get_org_access(channel_id_in in number, org_id_in in number)
    return number
    is
	throwaway number;
    begin
	-- the idea: if we get past this query,
        -- the org has access to the channel, else catch the exception and return 0
        select distinct 1 into throwaway
          from rhnChannelFamilyMembers CFM,
               rhnOrgChannelFamilyPermissions CFP
         where cfp.org_id = org_id_in
           and CFM.channel_family_id = CFP.channel_family_id
           and CFM.channel_id = channel_id_in
           and (CFP.max_members > 0 or CFP.max_members is null or CFP.org_id = 1);
        return 1;
        exception
            when no_data_found
            then
            return 0;
    end;
    function user_role_check_debug(channel_id_in in number,
                                   user_id_in in number,
                                   role_in in varchar2,
                                   reason_out out varchar2)
    return number
    is
	org_id number;
    begin
	org_id := rhn_user.get_org_id(user_id_in);
	if role_in = 'manage' and
           NVL(rhn_channel.get_org_id(channel_id_in), -1) <> org_id then
	        reason_out := 'channel_not_owned';
	       return 0;
	    end if;
	if role_in = 'subscribe' and
           rhn_channel.get_org_access(channel_id_in, org_id) = 0 then
	        reason_out := 'channel_not_available';
	        return 0;
	    end if;
	if rhn_user.check_role_implied(user_id_in, 'channel_admin') = 1 then
            reason_out := 'channel_admin';
            return 1;
	    end if;
	if role_in = 'subscribe'
	then
	    if rhn_channel.org_channel_setting(channel_id_in,
                       org_id,
                       'not_globally_subscribable') = 0 then
		reason_out := 'globally_subscribable';
		    return 1;
	    end if;
	end if;
	reason_out := 'direct_permission';
	return rhn_channel.direct_user_role_check(channel_id_in,
                                              user_id_in, role_in);
    end;
    function user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
	throwaway varchar2(256);
    begin
	return rhn_channel.user_role_check_debug(channel_id_in, user_id_in, role_in, throwaway);
    end;
    function loose_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
    begin
        if user_id_in is null then
            return 1;
        end if;
        return user_role_check(channel_id_in, user_id_in, role_in);
    end loose_user_role_check;
    function direct_user_role_check(channel_id_in in number, user_id_in in number, role_in in varchar2)
    return number
    is
	throwaway number;
    begin
	select 1 into throwaway
	  from rhnChannelPermissionRole CPR,
	       rhnChannelPermission CP
	 where CP.user_id = user_id_in
	   and CP.channel_id = channel_id_in
	   and CPR.label = role_in
	   and CP.role_id = CPR.id;
	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end;
    function org_channel_setting(channel_id_in in number, org_id_in in number, setting_in in varchar2)
    return number
    is
	throwaway number;
    begin
	select 1 into throwaway
	  from rhnOrgChannelSettingsType OCST,
	       rhnOrgChannelSettings OCS
	 where OCS.org_id = org_id_in
	   and OCS.channel_id = channel_id_in
	   and OCST.label = setting_in
	   and OCS.setting_id = OCST.id;
	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end;
    FUNCTION channel_priority(channel_id_in IN number)
    RETURN number
    IS
         channel_name varchar2(64);
         priority number;
         end_of_life_val date;
         org_id_val number;
    BEGIN
        select name, end_of_life, org_id
        into channel_name, end_of_life_val, org_id_val
        from rhnChannel
        where id = channel_id_in;
        if end_of_life_val is not null then
          return -400;
        end if;
        if channel_name like 'Red Hat Enterprise Linux%' or channel_name like 'RHEL%' then
          priority := 1000;
          if channel_name not like '%Beta%' then
            priority := priority + 1000;
          end if;
          priority := priority +
            case
              when channel_name like '%v. 5%' then 600
              when channel_name like '%v. 4%' then 500
              when channel_name like '%v. 3%' then 400
              when channel_name like '%v. 2%' then 300
              when channel_name like '%v. 1%' then 200
              else 0
            end;
          priority := priority +
            case
              when channel_name like 'Red Hat Enterprise Linux (v. 5%' then 60
              when (channel_name like '%AS%' and channel_name not like '%Extras%') then 50
              when (channel_name like '%ES%' and channel_name not like '%Extras%') then 40
              when (channel_name like '%WS%' and channel_name not like '%Extras%') then 30
              when (channel_name like '%Desktop%' and channel_name not like '%Extras%') then 20
              when channel_name like '%Extras%' then 10
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%)' then 5
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif channel_name like 'Red Hat Desktop%' then
            priority := 900;
            if channel_name not like '%Beta%' then
               priority := priority + 50;
            end if;
          priority := priority +
            case
              when channel_name like '%v. 4%' then 40
              when channel_name like '%v. 3%' then 30
              when channel_name like '%v. 2%' then 20
              when channel_name like '%v. 1%' then 10
              else 0
            end;
          priority := priority +
            case
              when channel_name like '%32-bit x86%' then 4
              when channel_name like '%64-bit Intel Itanium%' then 3
              when channel_name like '%64-bit AMD64/Intel EM64T%' then 2
              else 0
            end;
        elsif org_id_val is not null then
          priority := 600;
        else
          priority := 500;
        end if;
      return -priority;
    end channel_priority;
    procedure delete_server_channels(server_id_in in number)
    is
    begin
	update	rhnPrivateChannelFamily
	set	current_members = current_members -1
	where	org_id in (
			select	org_id
			from	rhnServer
			where	id = server_id_in
		)
		and channel_family_id in (
			select	rcfm.channel_family_id
			from	rhnChannelFamilyMembers rcfm,
				rhnServerChannel rsc
			where	rsc.server_id = server_id_in
				and rsc.channel_id = rcfm.channel_id
                and not exists (
                    select 1
                    from
                        rhnChannelFamilyVirtSubLevel cfvsl,
                        rhnSGTypeVirtSubLevel sgtvsl,
                        rhnServerEntitlementView sev,
                        rhnVirtualInstance vi
                    where
                        vi.virtual_system_id = server_id_in
                        and vi.host_system_id = sev.server_id
                        and sev.label in ('virtualization_host',
                                          'virtualization_host_platform')
                        and sev.server_group_type_id =
                            sgtvsl.server_group_type_id
                        and sgtvsl.virt_sub_level_id = cfvsl.virt_sub_level_id
                        and cfvsl.channel_family_id = rcfm.channel_family_id
                    )
		);
    end;
	procedure refresh_newest_package(channel_id_in in number, caller_in in varchar2 := '(unknown)')
	is
	begin
		delete from rhnChannelNewestPackage where channel_id = channel_id_in;
		insert into rhnChannelNewestPackage
			( channel_id, name_id, evr_id, package_id, package_arch_id )
			(	select	channel_id,
						name_id, evr_id,
						package_id, package_arch_id
				from	rhnChannelNewestPackageView
				where	channel_id = channel_id_in
			);
                insert into rhnChannelNewestPackageAudit (channel_id, caller)
                    values (channel_id_in, caller_in);
                update rhnChannel
                    set last_modified = greatest(sysdate, last_modified + 1/86400)
                    where id = channel_id_in;
	end;
   procedure update_channel ( channel_id_in in number, invalidate_ss in number := 0,
                              date_to_use in date := sysdate )
   is
   channel_last_modified date;
   last_modified_value date;
   cursor snapshots is
   select  snapshot_id id
   from    rhnSnapshotChannel
   where   channel_id = channel_id_in;
   begin
      select last_modified
      into channel_last_modified
      from rhnChannel
      where id = channel_id_in;
      last_modified_value := date_to_use;
      if last_modified_value <= channel_last_modified then
          last_modified_value := last_modified_value + 1/86400;
      end if;
      update rhnChannel set last_modified = last_modified_value
      where id = channel_id_in;
      if invalidate_ss = 1 then
        for snapshot in snapshots loop
            update rhnSnapshot
            set invalid = lookup_snapshot_invalid_reason('channel_modified')
            where id = snapshot.id;
        end loop;
      end if;
   end update_channel;
   procedure update_channels_by_package ( package_id_in in number, date_to_use in date := sysdate )
   is
   cursor channels is
   select channel_id
   from rhnChannelPackage
   where package_id = package_id_in
   order by channel_id;
   begin
      for channel in channels loop
         rhn_channel.update_channel ( channel.channel_id, 1, date_to_use );
      end loop;
   end update_channels_by_package;
   procedure update_channels_by_errata ( errata_id_in number, date_to_use in date := sysdate )
   is
   cursor channels is
   select channel_id
   from rhnChannelErrata
   where errata_id = errata_id_in
   order by channel_id;
   begin
      for channel in channels loop
         rhn_channel.update_channel ( channel.channel_id, 0, date_to_use );
      end loop;
   end update_channels_by_errata;
END rhn_channel;
/

SHOW ERRORS;

--
-- 17. RHN_CONFIG_CHANNEL  (Package Body)
--
--
-- RHN_CONFIG_CHANNEL  (Package Body)
--
CREATE OR REPLACE PACKAGE BODY rhn_config_channel
IS
    FUNCTION action_diff_revision_status(action_config_revision_id_in IN NUMBER)
    RETURN VARCHAR2
    IS
	failure_reason VARCHAR2(4000);
	result_is_null NUMBER;
	result_exists NUMBER;
    BEGIN
	select extant, is_null, reason
	into   result_exists, result_is_null, failure_reason
	from   (
		SELECT ACRR.action_config_revision_id extant,
		       1 is_null, -- NVL2(ACRR.result, 0, 1),
		       CFF.name reason
		  FROM rhnConfigFileFailure CFF,
		       rhnActionConfigRevisionResult ACRR,
		       rhnActionConfigRevision ACR
		 WHERE ACR.id = action_config_revision_id_in
		   AND ACR.id = ACRR.action_config_revision_id (+)
		   and acrr.result is null
		   AND ACR.failure_id = CFF.id (+)
		union all
		SELECT ACRR.action_config_revision_id extant,
		       0 is_null, -- NVL2(ACRR.result, 0, 1),
		       CFF.name reason
		  FROM rhnConfigFileFailure CFF,
		       rhnActionConfigRevisionResult ACRR,
		       rhnActionConfigRevision ACR
		 WHERE ACR.id = action_config_revision_id_in
		   AND ACR.id = ACRR.action_config_revision_id (+)
		   and acrr.result is not null
		   AND ACR.failure_id = CFF.id (+)
		);
	IF failure_reason IS NOT NULL
	THEN
	    RETURN failure_reason;
	END IF;
	IF result_exists IS NOT NULL
	THEN
	    IF result_is_null = 1
	    THEN
		RETURN 'No differences';
	    END IF;
	    RETURN 'Differences exist';
	END IF;
	RETURN NULL;
    END action_diff_revision_status;
    FUNCTION get_user_chan_access(config_channel_id_in IN NUMBER, user_id_in IN NUMBER)
    RETURN NUMBER
    IS
	server_id NUMBER;
	org_matches NUMBER;
	global_channel VARCHAR2(30);
	any_visible_servers_subscribed NUMBER;
    BEGIN
	org_matches := 0;
	BEGIN
	  SELECT 1 INTO org_matches
	    FROM rhnConfigChannel CC,
	         web_contact WC
	   WHERE WC.id = user_id_in
	     AND CC.id = config_channel_id_in
	     AND WC.org_id = CC.org_id;
	EXCEPTION
	  WHEN NO_DATA_FOUND
	    THEN RETURN 0;
	END;
	global_channel := 'unknown';
	SELECT CCT.label INTO global_channel
	  FROM rhnConfigChannel CC,
	       rhnConfigChannelType CCT
	 WHERE CC.id = config_channel_id_in
	   AND CCT.id = CC.confchan_type_id;
	IF (rhn_user.check_role_implied(user_id_in, 'config_admin') = 1) AND (global_channel = 'normal')
	THEN
	    RETURN 1;
	END IF;
	any_visible_servers_subscribed := 0;
	BEGIN
	  SELECT 1 INTO any_visible_servers_subscribed
	    FROM DUAL
           WHERE EXISTS (
	     SELECT SCC.server_id
	       FROM rhnServerConfigChannel SCC,
	            rhnUserServerPermsDupes USPD
	      WHERE USPD.user_id = user_id_in
	        AND USPD.server_id = SCC.server_id
	        AND SCC.config_channel_id = config_channel_id_in
	   );
	EXCEPTION
	  WHEN NO_DATA_FOUND
	    THEN RETURN 0;
	END;
	RETURN any_visible_servers_subscribed;
    END get_user_chan_access;
    FUNCTION get_user_revision_access(config_revision_id_in IN NUMBER, user_id_in IN NUMBER)
    RETURN NUMBER
    IS
	config_channel_id NUMBER;
    BEGIN
    BEGIN
	SELECT CF.config_channel_id INTO config_channel_id
	  FROM rhnConfigFile CF,
	       rhnConfigRevision CR
	 WHERE CF.id = CR.config_file_id
	   AND CR.id = config_revision_id_in;
    EXCEPTION
	  WHEN NO_DATA_FOUND
	   THEN RETURN 0;
    END;
	RETURN rhn_config_channel.get_user_chan_access(config_channel_id, user_id_in);
    END get_user_revision_access;
    FUNCTION get_user_file_access(config_file_id_in IN NUMBER, user_id_in IN NUMBER)
    RETURN NUMBER
    IS
	config_channel_id NUMBER;
    BEGIN
    BEGIN
        SELECT CF.config_channel_id INTO config_channel_id
	  FROM rhnConfigFile CF
	 WHERE CF.id = config_file_id_in;
	EXCEPTION
	  WHEN NO_DATA_FOUND
	    THEN RETURN 0;
    END;
	RETURN rhn_config_channel.get_user_chan_access(config_channel_id, user_id_in);
    END get_user_file_access;
END rhn_config_channel;
/

SHOW ERRORS;

--
-- 18. RHN_ENTITLEMENTS  (Package Body)
--
--
-- RHN_ENTITLEMENTS  (Package Body)
--


--
-- $Id: rhn_entitlements.pkb 137074 2008-02-19 10:31:42Z jpazdzio $
--

create or replace
package body rhn_entitlements
is
	body_version varchar2(100) := '$Id: rhn_entitlements.pkb 137074 2008-02-19 10:31:42Z jpazdzio $';


    -- *******************************************************************
    -- PROCEDURE: remove_org_entitlements
    --
    -- Removes both system entitlements and channel subscriptions
    -- that are currently assigned to an org and re-assigns to the
    -- master org (org_id = 1).
    --
    -- When we call this we expect everything to already be unentitled
    -- which shoul be handled by delete_org.
    --
    -- Called by: delete_org
    -- *******************************************************************
    procedure remove_org_entitlements(
        org_id_in in number
    )
    is

        cursor system_ents is
        select sg.id, sg.max_members, sg.group_type
        from rhnServerGroup sg
        where group_type is not null
          and org_id = org_id_in;

        cursor channel_subs is
        select pcf.channel_family_id, pcf.max_members
        from rhnChannelFamily cf,
             rhnPrivateChannelFamily pcf
        where pcf.org_id = org_id_in
          and pcf.channel_family_id = cf.id
          and cf.org_id is null;

    begin

        for system_ent in system_ents loop
            update rhnServerGroup
            set max_members = max_members + system_ent.max_members
            where org_id = 1
              and group_type = system_ent.group_type;
        end loop;

        update rhnServerGroup
        set max_members = 0
        where org_id = org_id_in;

        for channel_sub in channel_subs loop
            update rhnPrivateChannelFamily
            set max_members = max_members + channel_sub.max_members
            where org_id = 1
              and channel_family_id = channel_sub.channel_family_id;
        end loop;

        update rhnPrivateChannelFamily
        set max_members = 0
        where org_id = org_id_in;

    end remove_org_entitlements;

	function entitlement_grants_service (
	    entitlement_in in varchar2,
	    service_level_in in varchar2
	) return number	is
	begin
		if service_level_in = 'provisioning' then
			if entitlement_in = 'provisioning_entitled' then
				return 1;
			else
				return 0;
			end if;
		elsif service_level_in = 'management' then
			if entitlement_in = 'enterprise_entitled' then
				return 1;
			else
				return 0;
			end if;
		elsif service_level_in = 'monitoring' then
			if entitlement_in = 'monitoring_entitled' then
				return 1;
			end if;
		elsif service_level_in = 'updates' then
			return 1;
		else
			return 0;
		end if;
	end entitlement_grants_service;

	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		cursor server_groups is
			select	sg.id				server_group_id
			from	rhnServerGroup		sg,
					rhnServerGroupType	sgt
			where	sgt.label = type_label_in
				and sgt.id = sg.group_type
				and sg.org_id = org_id_in;
	begin
		for sg in server_groups loop
			return sg.server_group_id;
		end loop;
		return rhn_entitlements.create_entitlement_group(
				org_id_in,
				type_label_in
			);
	end lookup_entitlement_group;

	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number is
		sg_id_val number;
	begin
		select	rhn_server_group_id_seq.nextval
		into	sg_id_val
		from	dual;

		insert into rhnServerGroup (
				id, name, description, max_members, current_members,
				group_type, org_id
			) (
				select	sg_id_val, sgt.label, sgt.label,
						0, 0, sgt.id, org_id_in
				from	rhnServerGroupType sgt
				where	sgt.label = type_label_in
			);

		return sg_id_val;
	end create_entitlement_group;

   function can_entitle_server (
        server_id_in in number,
        type_label_in in varchar2 )
   return number is
      cursor addon_servergroups (base_label_in in varchar2,
                                 addon_label_in in varchar2) is
         select
            addon_id
         from
            rhnSGTypeBaseAddonCompat
         where base_id = lookup_sg_type (base_label_in)
           and addon_id = lookup_sg_type (addon_label_in);

      previous_ent        rhn_entitlements.ents_array;
      is_base_in          char   := 'N';
      is_base_current     char   := 'N';
      i                   number := 0;
      sgid                number := 0;

   begin

      previous_ent := rhn_entitlements.ents_array();
      previous_ent := rhn_entitlements.get_server_entitlement(server_id_in);

      select distinct is_base
      into is_base_in
      from rhnServerGroupType
      where label = type_label_in;

      if previous_ent.count = 0 then
         if (is_base_in = 'Y' and rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid)) then
            -- rhn_server.insert_into_servergroup (server_id_in, sgid);
            return 1;
         else
            -- rhn_exception.raise_exception ('invalid_base_entitlement');
            return 0;
         end if;

      -- there are previous ents, first make sure we're not trying to entitle a base ent
      elsif is_base_in = 'Y' then
         -- rhn_exception.raise_exception ('invalid_addon_entitlement');
         return 0;

      -- it must be an addon, so proceed with the entitlement
      else

         -- find the servers base ent
         is_base_current := 'N';
         i := 0;
         while is_base_current = 'N' and i <= previous_ent.count
         loop
            i := i + 1;
            select is_base
            into is_base_current
            from rhnServerGroupType
            where label = previous_ent(i);
         end loop;

         -- never found a base ent, that would be strange
         if is_base_current  = 'N' then
            -- rhn_exception.raise_exception ('invalid_base_entitlement');
            return 0;
         end if;

         -- this for loop verifies the validity of the addon path
         for addon_servergroup in addon_servergroups  (previous_ent(i), type_label_in) loop
            -- find an appropriate sgid for the addon and entitle the server
            if rhn_entitlements.find_compatible_sg (server_id_in, type_label_in, sgid) then
               -- rhn_server.insert_into_servergroup (server_id_in, sgid);
               return 1;
            else
               -- rhn_exception.raise_exception ('invalid_addon_entitlement');
               return 0;
            end if;
         end loop;

      end if;

      return 0;

   end can_entitle_server;

   function can_switch_base (
      server_id_in   in    integer,
      type_label_in  in    varchar2
   ) return number is

      type_label_in_is_base   char(1);
      sgid                    number;

   begin

      begin
         select is_base into type_label_in_is_base
         from rhnServerGroupType
         where label = type_label_in;
      exception
         when no_data_found then
            rhn_exception.raise_exception ( 'invalid_entitlement' );
      end;

      if type_label_in_is_base = 'N' then
         rhn_exception.raise_exception ( 'invalid_entitlement' );
      elsif rhn_entitlements.find_compatible_sg ( server_id_in,
                                                  type_label_in, sgid ) then
         return 1;
      else
         return 0;
      end if;

   end can_switch_base;


   function find_compatible_sg (
      server_id_in    in   number,
      type_label_in   in   varchar2,
      sgid_out        out  number
   ) return boolean is

      cursor servergroups is
         select sg.id            id
           from rhnServerGroupType             sgt,
                rhnServerGroup                 sg,
                rhnServer                     s,
                rhnServerServerGroupArchCompat ssgac
          where s.id = server_id_in
            and s.org_id = sg.org_id
            and sgt.label = type_label_in
            and sg.group_type = sgt.id
            and ssgac.server_group_type = sgt.id
            and ssgac.server_arch_id = s.server_arch_id
            and not exists (
                     select 1
                      from rhnServerGroupMembers sgm
                     where sgm.server_group_id = sg.id
                       and sgm.server_id = s.id);


   begin
      for servergroup in servergroups loop
         sgid_out := servergroup.id;
         return true;
      end loop;

      --no servergroup found
      sgid_out := 0;
      return false;
   end find_compatible_sg;

	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) is
      sgid  number := 0;
      is_virt number := 0;

	begin

          begin
          select 1 into is_virt
            from rhnServerEntitlementView
           where server_id = server_id_in
             and label in ('virtualization_host', 'virtualization_host_platform');
	  exception
            when no_data_found then
              is_virt := 0;
          end;

      if is_virt = 0 and (type_label_in = 'virtualization_host' or
                          type_label_in = 'virtualization_host_platform') then

        is_virt := 1;
      end if;



      if rhn_entitlements.can_entitle_server(server_id_in,
                                             type_label_in) = 1 then
         if rhn_entitlements.find_compatible_sg (server_id_in,
                                                 type_label_in, sgid) then
            insert into rhnServerHistory ( id, server_id, summary, details )
            values ( rhn_event_id_seq.nextval, server_id_in,
                     'added system entitlement ',
                      case type_label_in
                       when 'enterprise_entitled' then 'Management'
                       when 'sw_mgr_entitled' then 'Update'
                       when 'provisioning_entitled' then 'Provisioning'
                       when 'monitoring_entitled' then 'Monitoring'
                       when 'virtualization_host' then 'Virtualization'
                       when 'virtualization_host_platform' then
                            'Virtualization Platform' end  );

            rhn_server.insert_into_servergroup (server_id_in, sgid);

            if is_virt = 1 then
              rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
            end if;

         else
            rhn_exception.raise_exception ('no_available_server_group');
         end if;
      else
         rhn_exception.raise_exception ('invalid_entitlement');
      end if;
   end entitle_server;

	procedure remove_server_entitlement (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled',
        repoll_virt_guests in number := 1
	) is
		group_id number;
      type_is_base char;
      is_virt number := 0;
	begin
      begin


      -- would be nice if there were a virt attribute of entitlement types, not have to specify 2 different ones...
        begin
          select 1 into is_virt
            from rhnServerEntitlementView
           where server_id = server_id_in
             and label in ('virtualization_host', 'virtualization_host_platform');
        exception
          when no_data_found then
            is_virt := 0;
        end;

		select	sg.id, sgt.is_base
		into	group_id, type_is_base
		from	rhnServerGroupType sgt,
			rhnServerGroup sg,
				rhnServerGroupMembers sgm,
				rhnServer s
		where	s.id = server_id_in
			and s.id = sgm.server_id
			and sgm.server_group_id = sg.id
			and sg.org_id = s.org_id
			and sgt.label = type_label_in
			and sgt.id = sg.group_type;

      if ( type_is_base = 'Y' ) then
         -- unentitle_server should handle everything, don't really need to do anything else special here
         unentitle_server ( server_id_in );
      else

         insert into rhnServerHistory ( id, server_id, summary, details )
         values ( rhn_event_id_seq.nextval, server_id_in,
                  'removed system entitlement ',
                   case type_label_in
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'
                    when 'virtualization_host' then 'Virtualization'
                    when 'virtualization_host_platform' then
                         'Virtualization Platforrm' end  );

         rhn_server.delete_from_servergroup(server_id_in, group_id);

         -- special case: clean up related monitornig data
         if type_label_in = 'monitoring_entitled' then
           DELETE
             FROM state_change
            WHERE o_id IN (SELECT probe_id
                             FROM rhn_check_probe
                            WHERE host_id = server_id_in);
           DELETE
             FROM time_series
            WHERE SUBSTR(o_id, INSTR(o_id, '-') + 1,
                        (INSTR(o_id, '-', INSTR(o_id, '-') + 1) - INSTR(o_id, '-')) - 1)
              IN (SELECT probe_id
                    FROM rhn_check_probe
                   WHERE host_id = server_id_in);
           DELETE
             FROM rhn_probe
            WHERE recid IN (SELECT probe_id
                              FROM rhn_check_probe
                             WHERE host_id = server_id_in);
         end if;

         if is_virt = 1 and repoll_virt_guests = 1 then
           rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
         end if;
      end if;

		exception
		when no_data_found then
				rhn_exception.raise_exception('invalid_server_group_member');
      end;

	end remove_server_entitlement;


   procedure unentitle_server (server_id_in in number) is

      cursor servergroups is
         select distinct sgt.label, sg.id server_group_id
         from  rhnServerGroupType sgt,
               rhnServerGroup sg,
               rhnServer s,
               rhnServerGroupMembers sgm
         where s.id = server_id_in
            and s.org_id = sg.org_id
            and sg.group_type = sgt.id
            and sgm.server_group_id = sg.id
            and sgm.server_id = s.id;

     is_virt number := 0;

   begin

      begin
        select 1 into is_virt
          from rhnServerEntitlementView
         where server_id = server_id_in
           and label in ('virtualization_host', 'virtualization_host_platform');
      exception
        when no_data_found then
          is_virt := 0;
      end;

      for servergroup in servergroups loop

         insert into rhnServerHistory ( id, server_id, summary, details )
         values ( rhn_event_id_seq.nextval, server_id_in,
                  'removed system entitlement ',
                   case servergroup.label
                    when 'enterprise_entitled' then 'Management'
                    when 'sw_mgr_entitled' then 'Update'
                    when 'provisioning_entitled' then 'Provisioning'
                    when 'monitoring_entitled' then 'Monitoring'
                    when 'virtualization_host' then 'Virtualization'
                    when 'virtualization_host_platform' then
                         'Virtualization Platform' end  );

         rhn_server.delete_from_servergroup(server_id_in,
                                            servergroup.server_group_id );
      end loop;

      if is_virt = 1 then
        rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
      end if;

   end unentitle_server;


    -- *******************************************************************
    -- PROCEDURE: repoll_virt_guest_entitlements
    --
    --   Whenever we add/remove a virtualization_host* entitlement from
    --   a host, we can call this procedure to update what type of slots
    --   the guests are consuming.
    --
    --   If you're removing the entitlement, it's
    --   possible the guests will become unentitled if you don't have enough
    --   physical slots to cover them.
    --
    --   If you're adding the entitlement, you end up freeing up physical
    --   slots for other systems.
    --
    -- *******************************************************************
    procedure repoll_virt_guest_entitlements(server_id_in in number)
    is

        -- All channel families associated with the guests of server_id_in
        cursor families is
            select distinct cfs.channel_family_id
            from
                rhnChannelFamilyServers cfs,
                rhnVirtualInstance vi
            where
                vi.host_system_id = server_id_in
                and vi.virtual_system_id = cfs.server_id;

        -- All of server group types associated with the guests of
        -- server_id_in
        cursor group_types is
            select distinct sg.group_type, sgt.label
            from
                rhnServerGroupType sgt,
                rhnServerGroup sg,
                rhnServerGroupMembers sgm,
                rhnVirtualInstance vi
            where
                vi.host_system_id = server_id_in
                and vi.virtual_system_id = sgm.server_id
                and sgm.server_group_id = sg.id
                and sg.group_type = sgt.id;

        -- Virtual servers from a certain family belonging to a speicifc
        -- host that are consuming physical channel slots over the limit.
        cursor virt_servers_cfam(family_id_in in number, quantity_in in number)  is
            select virtual_system_id
            from (
                select rownum, vi.virtual_system_id
                from
                    rhnChannelFamilyMembers cfm,
                    rhnServerChannel sc,
                    rhnVirtualInstance vi
                where
                    vi.host_system_id = server_id_in
                    and vi.virtual_system_id = sc.server_id
                    and sc.channel_id = cfm.channel_id
                    and cfm.channel_family_id = family_id_in
                order by sc.modified desc
                )
            where rownum <= quantity_in;

        -- Virtual servers from a certain family belonging to a speicifc
        -- host that are consuming physical system slots over the limit.
        cursor virt_servers_sgt(group_type_in in number, quantity_in in number)  is
            select virtual_system_id
            from (
                select rownum, vi.virtual_system_id
                from
                    rhnServerGroup sg,
                    rhnServerGroupMembers sgm,
                    rhnVirtualInstance vi
                where
                    vi.host_system_id = server_id_in
                    and vi.virtual_system_id = sgm.server_id
                    and sgm.server_group_id = sg.id
                    and sg.group_type = group_type_in
                order by sgm.modified desc
                )
            where rownum <= quantity_in;

        org_id_val number;
        max_members_val number;
        current_members_calc number;
        sg_id number;

    begin

        select org_id
        into org_id_val
        from rhnServer
        where id = server_id_in;

        -- deal w/ channel entitlements first ...
        for family in families loop
            -- get the current (physical) members of the family
            current_members_calc :=
                rhn_channel.channel_family_current_members(family.channel_family_id,
                                                           org_id_val); -- fixed transposed args

            -- get the max members of the family
            select max_members
            into max_members_val
            from rhnPrivateChannelFamily
            where channel_family_id = family.channel_family_id
            and org_id = org_id_val;

            if current_members_calc > max_members_val then
                -- A virtualization_host* ent must have been removed, so we'll
                -- unsubscribe guests from the host first.

                -- hm, i don't think max_members - current_members_calc yielding a negative number
                -- will work w/ rownum, swaping 'em in the body of this if...
                for virt_server in virt_servers_cfam(family.channel_family_id,
                                current_members_calc - max_members_val) loop

                    rhn_channel.unsubscribe_server_from_family(
                                virt_server.virtual_system_id,
                                family.channel_family_id);
                end loop;

                -- if we're still over the limit, which would be odd,
                -- just prune the group to max_members
                --
                -- er... wouldn't we actually have to refresh the values of
                -- current_members_calc and max_members_val to actually ever
                -- *skip this??
                if current_members_calc > max_members_val then
                    -- argh, transposed again?!
                    set_family_count(org_id_val,
                                     family.channel_family_id,
                                     max_members_val);
                end if;

           end if;

            -- update current_members for the family.  This will set the value
            -- to reflect adding/removing the entitlement.
            --
            -- what's the difference of doing this vs the unavoidable set_family_count above?
            rhn_channel.update_family_counts(family.channel_family_id,
                                             org_id_val);
        end loop;

        for a_group_type in group_types loop
          -- get the current *physical* members of the system entitlement type for the org...
          --
          -- unlike channel families, it appears the standard rhnServerGroup.max_members represents
          -- *physical* slots, vs physical+virt ... boy that's confusing...

          select max_members, id
            into max_members_val, sg_id
            from rhnServerGroup
            where group_type = a_group_type.group_type
            and org_id = org_id_val;


	  select count(sep.server_id) into current_members_calc
            from rhnServerEntitlementPhysical sep
           where sep.server_group_id = sg_id
             and sep.server_group_type_id = a_group_type.group_type;

          if current_members_calc > max_members_val then
            -- A virtualization_host* ent must have been removed, and we're over the limit, so unsubscribe guests
            for virt_server in virt_servers_sgt(a_group_type.group_type,
                                                current_members_calc - max_members_val) loop
              rhn_entitlements.remove_server_entitlement(virt_server.virtual_system_id, a_group_type.label);

              -- decrement current_members_calc, we'll use it to reset current_members for the group at the end...
              current_members_calc := current_members_calc - 1;
            end loop;

          end if;

          update rhnServerGroup set current_members = current_members_calc
           where org_id = org_id_val
             and group_type = a_group_type.group_type;

          -- I think that's all the house-keeping we have to do...
        end loop;

    end repoll_virt_guest_entitlements;


	function get_server_entitlement (
		server_id_in in number
	) return ents_array is

		cursor server_groups is
			select	sgt.label
			from	rhnServerGroupType		sgt,
					rhnServerGroup			sg,
					rhnServerGroupMembers	sgm
			where	1=1
				and sgm.server_id = server_id_in
				and sg.id = sgm.server_group_id
				and sgt.id = sg.group_type
				and sgt.label in (
					'sw_mgr_entitled','enterprise_entitled',
					'provisioning_entitled', 'nonlinux_entitled',
					'monitoring_entitled', 'virtualization_host',
                                        'virtualization_host_platform'
					);

         ent_array ents_array;

	begin

      ent_array := ents_array();

		for sg in server_groups loop
         ent_array.extend;
         ent_array(ent_array.count) := sg.label;
		end loop;

		return ent_array;

	end get_server_entitlement;


	-- this desperately needs to be table driven.
	procedure modify_org_service (
		org_id_in in number,
		service_label_in in varchar2,
		enable_in in char
	) is
		type roles_v is varray(10) of rhnUserGroupType.label%TYPE;
		roles_to_process roles_v;
		cursor roles(role_label_in in varchar2) is
			select	label, id
			from	rhnUserGroupType
			where	label = role_label_in;
		cursor org_roles(role_label_in in varchar2) is
			select	1
			from	rhnUserGroup ug,
					rhnUserGroupType ugt
			where	ugt.label = role_label_in
				and ug.org_id = org_id_in
				and ugt.id = ug.group_type;

		type ents_v is varray(10) of rhnOrgEntitlementType.label%TYPE;
		ents_to_process ents_v;
		cursor ents(ent_label_in in varchar2) is
			select	label, id
			from	rhnOrgEntitlementType
			where	label = ent_label_in;
		cursor org_ents(ent_label_in in varchar2) is
			select	1
			from	rhnOrgEntitlements oe,
					rhnOrgEntitlementType oet
			where	oet.label = ent_label_in
				and oe.org_id = org_id_in
				and oet.id = oe.entitlement_id;
		create_row char(1);
	begin
		ents_to_process := ents_v();
		roles_to_process := roles_v();
		-- a bit kludgy, but only for 3.4 really.  Certainly no
		-- worse than the old code...
		if service_label_in = 'enterprise' or
           service_label_in = 'management' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'org_admin';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'system_group_admin';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'activation_key_admin';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'org_applicant';
		elsif service_label_in = 'provisioning' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_provisioning';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'system_group_admin';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'activation_key_admin';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'config_admin';
			-- another nasty special case...
			if enable_in = 'Y' then
				ents_to_process.extend;
				ents_to_process(ents_to_process.count) := 'sw_mgr_enterprise';
			end if;
		elsif service_label_in = 'monitoring' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_monitor';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'monitoring_admin';
		elsif service_label_in = 'virtualization' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_virtualization';

			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'config_admin';
        elsif service_label_in = 'virtualization_platform' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_virtualization_platform';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'config_admin';
	elsif service_label_in = 'nonlinux' then
			ents_to_process.extend;
			ents_to_process(ents_to_process.count) := 'rhn_nonlinux';
			roles_to_process.extend;
			roles_to_process(roles_to_process.count) := 'config_admin';
		end if;

		if enable_in = 'Y' then
			for i in 1..ents_to_process.count loop
				for ent in ents(ents_to_process(i)) loop
					create_row := 'Y';
					for oe in org_ents(ent.label) loop
						create_row := 'N';
					end loop;
					if create_row = 'Y' then
						insert into rhnOrgEntitlements(org_id, entitlement_id)
							values (org_id_in, ent.id);
					end if;
				end loop;
			end loop;
			for i in 1..roles_to_process.count loop
				for role in roles(roles_to_process(i)) loop
					create_row := 'Y';
					for o_r in org_roles(role.label) loop
						create_row := 'N';
					end loop;
					if create_row = 'Y' then
						insert into rhnUserGroup(
								id, name, description, current_members,
								group_type, org_id
							) (
								select	rhn_user_group_id_seq.nextval,
										ugt.name || 's',
										ugt.name || 's for Org ' ||
											o.name || ' ('|| o.id ||')',
										0, ugt.id, o.id
								from	rhnUserGroupType ugt,
										web_customer o
								where	o.id = org_id_in
									and ugt.id = role.id
							);
					end if;
				end loop;
			end loop;
		else
			for i in 1..ents_to_process.count loop
				for ent in ents(ents_to_process(i)) loop
					delete from rhnOrgEntitlements
					 where org_id = org_id_in
					   and entitlement_id = ent.id;
				end loop;
			end loop;
		end if;
	end modify_org_service;

	procedure set_customer_enterprise (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'enterprise', 'Y');
	end set_customer_enterprise;

	procedure set_customer_provisioning (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'provisioning', 'Y');
	end set_customer_provisioning;

	procedure set_customer_monitoring (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'monitoring', 'Y');
	end set_customer_monitoring;

	procedure set_customer_nonlinux (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'nonlinux', 'Y');
	end set_customer_nonlinux;

	procedure unset_customer_enterprise (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'enterprise', 'N');
	end unset_customer_enterprise;

	procedure unset_customer_provisioning (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'provisioning', 'N');
	end unset_customer_provisioning;

	procedure unset_customer_monitoring (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'monitoring', 'N');
	end unset_customer_monitoring;

	procedure unset_customer_nonlinux (
		customer_id_in in number
	) is
	begin
		modify_org_service(customer_id_in, 'nonlinux', 'N');
	end unset_customer_nonlinux;

    -- *******************************************************************
    -- PROCEDURE: prune_group
    -- Unsubscribes servers consuming physical slots that over the org's
    --   limit.
    -- Called by: set_group_count, prune_everything, repoll_virt_guest_entitlements
    -- *******************************************************************
	procedure prune_group (
		group_id_in in number,
		type_in in char,
		quantity_in in number
	) is
		cursor usergroups is
			select	user_id, user_group_id, ugt.label
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	1=1
				and ugm.user_group_id = group_id_in
				and ugm.user_id in (
					select	user_id
					from	(
						select	rownum row_number,
								user_id,
								time
						from	(
							select	user_id,
									modified time
							from	rhnUserGroupMembers
							where	user_group_id = group_id_in
							order by time asc
						)
					)
					where	row_number > quantity_in
				)
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id;
        cursor servergroups is
           select  server_id, server_group_id, sgt.id group_type_id, sgt.label
            from    rhnServerGroupType              sgt,
                            rhnServerGroup                  sg,
                            rhnServerGroupMembers   sgm
            where   1=1
                    and sgm.server_group_id = group_id_in
                    and sgm.server_id in (
                            select  server_id
                            from    (
                                    select  rownum row_number,
                                                    server_id,
                                                    time
                                    from    (
                                            select  sep.server_id,
                                                    sep.modified time
                                            from
                                                rhnServerEntitlementPhysical sep
                                            where
                                                sep.server_group_id = group_id_in
                                            order by time asc
                                    )
                            )
                            where   row_number > quantity_in
                    )
                    and sgm.server_group_id = sg.id
                    and sg.group_type = sgt.id;
      type_is_base char;
	begin
		if type_in = 'U' then
			update		rhnUserGroup
				set		max_members = quantity_in
				where	id = group_id_in;

			for ug in usergroups loop
				rhn_user.remove_from_usergroup(ug.user_id, ug.user_group_id);
			end loop;
		elsif type_in = 'S' then
			update		rhnServerGroup
				set		max_members = quantity_in
				where	id = group_id_in;

			for sg in servergroups loop
				remove_server_entitlement(sg.server_id, sg.label);

            select is_base
            into type_is_base
            from rhnServerGroupType sgt
            where sgt.id = sg.group_type_id;

            -- if we're removing a base ent, then be sure to
            -- remove the server's channel subscriptions.
            if ( type_is_base = 'Y' ) then
				   rhn_channel.clear_subscriptions(sg.server_id);
            end if;

			end loop;
		end if;
	end prune_group;

    -- *******************************************************************
    -- PROCEDURE: assign_system_entitlement
    --
    -- Moves system entitlements from from_org_id_in to to_org_id_in.
    -- Can raise not_enough_entitlements_in_base_org if from_org_id_in
    -- does not have enough entitlements to cover the move.
    -- Takes care of unentitling systems if necessary by calling
    -- set_group_count
    -- *******************************************************************
    procedure assign_system_entitlement(
        group_label_in in varchar2,
        from_org_id_in in number,
        to_org_id_in in number,
        quantity_in in number
    )
    is
        prev_ent_count number;
	to_org_prev_ent_count number;
        new_ent_count number;
	new_quantity number;
        group_type number;
    begin

        begin
            select max_members
            into prev_ent_count
            from rhnServerGroupType sgt,
                 rhnServerGroup sg
            where sg.org_id = from_org_id_in
              and sg.group_type = sgt.id
              and sgt.label = group_label_in;
        exception
            when NO_DATA_FOUND then
                rhn_exception.raise_exception(
                              'not_enough_entitlements_in_base_org');
        end;

        begin
            select max_members
            into to_org_prev_ent_count
            from rhnServerGroupType sgt,
                 rhnServerGroup sg
            where sg.org_id = to_org_id_in
              and sg.group_type = sgt.id
              and sgt.label = group_label_in;
        exception
            when NO_DATA_FOUND then
                to_org_prev_ent_count := 0;
        end;

        begin
            select id
            into group_type
            from rhnServerGroupType
            where label = group_label_in;
        exception
            when NO_DATA_FOUND then
                rhn_exception.raise_exception(
                              'invalid_server_group');
        end;

        new_ent_count := prev_ent_count - quantity_in;

        if prev_ent_count > new_ent_count then
            new_quantity := to_org_prev_ent_count + quantity_in;
        end if;

        if new_ent_count < 0 then
            rhn_exception.raise_exception(
                          'not_enough_entitlements_in_base_org');
        end if;


        rhn_entitlements.set_group_count(from_org_id_in,
                                         'S',
                                         group_type,
                                         new_ent_count);

        rhn_entitlements.set_group_count(to_org_id_in,
                                         'S',
                                         group_type,
                                         new_quantity);

        -- Create or delete the entries in rhnOrgEntitlementType
        if group_label_in = 'enterprise_entitled' then
            if new_quantity > 0 then
                set_customer_enterprise(to_org_id_in);
            else
                unset_customer_enterprise(to_org_id_in);
            end if;
        end if;

        if group_label_in = 'provisioning_entitled' then
            if new_quantity > 0 then
                set_customer_provisioning(to_org_id_in);
            else
                unset_customer_provisioning(to_org_id_in);
            end if;
        end if;

        if group_label_in = 'monitoring_entitled' then
            if new_quantity > 0 then
                set_customer_monitoring(to_org_id_in);
            else
                unset_customer_monitoring(to_org_id_in);
            end if;
        end if;

    end assign_system_entitlement;

    -- *******************************************************************
    -- PROCEDURE: assign_channel_entitlement
    --
    -- Moves channel entitlements from from_org_id_in to to_org_id_in.
    -- Can raise not_enough_entitlements_in_base_org if from_org_id_in
    -- does not have enough entitlements to cover the move.
    -- Takes care of unentitling systems if necessary by calling
    -- set_family_count
    -- *******************************************************************
    procedure assign_channel_entitlement(
        channel_family_label_in in varchar2,
        from_org_id_in in number,
        to_org_id_in in number,
        quantity_in in number
    )
    is
        prev_ent_count number;
        new_ent_count number;
	to_org_prev_ent_count number;
	new_quantity number;
        cfam_id       number;
    begin

        begin
            select max_members
            into prev_ent_count
            from rhnChannelFamily cf,
                 rhnPrivateChannelFamily pcf
            where pcf.org_id = from_org_id_in
              and pcf.channel_family_id = cf.id
              and cf.label = channel_family_label_in;
        exception
            when NO_DATA_FOUND then
                rhn_exception.raise_exception(
                              'not_enough_entitlements_in_base_org');
        end;

        begin
            select max_members
            into to_org_prev_ent_count
            from rhnChannelFamily cf,
                 rhnPrivateChannelFamily pcf
            where pcf.org_id = to_org_id_in
              and pcf.channel_family_id = cf.id
              and cf.label = channel_family_label_in;
        exception
            when NO_DATA_FOUND then
                to_org_prev_ent_count := 0;
        end;


        begin
            select id
            into cfam_id
            from rhnChannelFamily
            where label = channel_family_label_in;
        exception
            when NO_DATA_FOUND then
                rhn_exception.raise_exception(
                              'invalid_channel_family');
        end;

        new_ent_count := prev_ent_count - quantity_in;

	if prev_ent_count > new_ent_count then
            new_quantity := to_org_prev_ent_count + quantity_in;
	end if;


        if new_ent_count < 0 then
            rhn_exception.raise_exception(
                          'not_enough_entitlements_in_base_org');
        end if;

        rhn_entitlements.set_family_count(from_org_id_in,
                                          cfam_id,
                                          new_ent_count);

        rhn_entitlements.set_family_count(to_org_id_in,
                                          cfam_id,
                                          new_quantity);

    end assign_channel_entitlement;

    -- *******************************************************************
    -- PROCEDURE: activate_system_entitlement
    --
    -- Sets the values in rhnServerGroup for a given rhnServerGroupType.
    --
    -- Calls: set_group_count to update, prune, or create the group.
    -- Called by: the code that activates a satellite cert.
    --
    -- Raises not_enough_entitlements_in_base_org if all entitlements
    -- in the org are used so the free entitlements would not cover
    -- the difference when descreasing the number of entitlements.
    -- *******************************************************************
    procedure activate_system_entitlement(
        org_id_in in number,
        group_label_in in varchar2,
        quantity_in in number
    )
    is
        prev_ent_count number;
        prev_ent_count_sum number;
        group_type number;
    begin

        -- Fetch the current entitlement count for the org
        -- into prev_ent_count
        begin
            select current_members
            into prev_ent_count
            from rhnServerGroupType sgt,
                 rhnServerGroup sg
            where sg.group_type = sgt.id
              and sgt.label = group_label_in
              and sg.org_id = org_id_in;
        exception
            when NO_DATA_FOUND then
                prev_ent_count := 0;
        end;

        begin
            select id
            into group_type
            from rhnServerGroupType
            where label = group_label_in;
        exception
            when NO_DATA_FOUND then
                rhn_exception.raise_exception(
                              'invalid_server_group');
        end;

        -- If we're setting the total entitlemnt count to a lower value,
        -- and that value is less than the allocated count in this org,
        -- we need to raise an exception.
        if quantity_in < prev_ent_count then
            rhn_exception.raise_exception(
                          'not_enough_entitlements_in_base_org');
        else
            rhn_entitlements.set_group_count(org_id_in,
                                             'S',
                                             group_type,
                                             quantity_in);
        end if;


    end activate_system_entitlement;

    -- *******************************************************************
    -- PROCEDURE: activate_channel_entitlement
    --
    -- Calls: set_family_count to update, prune, or create the family
    --        permission bucket.
    -- Called by: the code that activates a satellite cert.
    --
    -- Raises not_enough_entitlements_in_base_org if there are not enough
    -- entitlements in the org to cover the difference when you are
    -- descreasing the number of entitlements.
    --
    -- The backend code in Python is expected to do whatever arithmetics
    -- is needed.
    -- *******************************************************************
    procedure activate_channel_entitlement(
        org_id_in in number,
        channel_family_label_in in varchar2,
        quantity_in in number
    )
    is
        prev_ent_count number;
        prev_ent_count_sum number;
        cfam_id number;
    begin

        -- Fetch the current entitlement count for the org
        -- into prev_ent_count
        begin
            select current_members
            into prev_ent_count
            from rhnChannelFamily cf,
                 rhnPrivateChannelFamily pcf
            where pcf.org_id = org_id_in
              and pcf.channel_family_id = cf.id
              and cf.label = channel_family_label_in;
        exception
            when NO_DATA_FOUND then
                prev_ent_count := 0;
        end;

        begin
            select id
            into cfam_id
            from rhnChannelFamily
            where label = channel_family_label_in;
        exception
            when NO_DATA_FOUND then
                rhn_exception.raise_exception(
                              'invalid_channel_family');
        end;

        -- If we're setting the total entitlemnt count to a lower value,
        -- and that value is less than the count in that one org,
        -- we need to raise an exception.
        if quantity_in < prev_ent_count then
            rhn_exception.raise_exception(
                          'not_enough_entitlements_in_base_org');
        else
            rhn_entitlements.set_family_count(org_id_in,
                                              cfam_id,
                                              quantity_in);
        end if;

    end activate_channel_entitlement;


	procedure set_group_count (
		customer_id_in in number,
		type_in in char,
		group_type_in in number,
		quantity_in in number
	) is
		group_id number;
		quantity number;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;

		if type_in = 'U' then
			select	rug.id
			into	group_id
			from	rhnUserGroup rug
			where	1=1
				and rug.org_id = customer_id_in
				and rug.group_type = group_type_in;
		elsif type_in = 'S' then
			select	rsg.id
			into	group_id
			from	rhnServerGroup rsg
			where	1=1
				and rsg.org_id = customer_id_in
				and rsg.group_type = group_type_in;
		end if;

		rhn_entitlements.prune_group(
			group_id,
			type_in,
			quantity
		);
	exception
		when no_data_found then
			if type_in = 'U' then
				insert into rhnUserGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_user_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnUserGroupType
						where	id = group_type_in
				);
			elsif type_in = 'S' then
				insert into rhnServerGroup (
						id, name, description, max_members, current_members,
						group_type, org_id, created, modified
					) (
						select	rhn_server_group_id_seq.nextval, name, name,
								quantity, 0, id, customer_id_in,
								sysdate, sysdate
						from	rhnServerGroupType
						where	id = group_type_in
				);
			end if;
	end set_group_count;

    -- *******************************************************************
    -- PROCEDURE: prune_family
    -- Unsubscribes servers consuming physical slots from the channel family
    --   that are over the org's limit.
    -- Called by: set_family_count, prune_everything
    -- *******************************************************************
	procedure prune_family (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor serverchannels is
			select	sc.server_id,
					sc.channel_id
			from	rhnServerChannel sc,
					rhnChannelFamilyMembers cfm
			where	1=1
				and cfm.channel_family_id = channel_family_id_in
				and cfm.channel_id = sc.channel_id
				and server_id in (
					select	server_id
					from	(
						select	server_id,
								time,
								rownum row_number
						from	(
							select	rs.id					server_id,
									rcfm.modified			time
							from
									rhnServerChannel		rsc,
									rhnChannelFamilyMembers	rcfm,
                                    rhnServer				rs
							where	1=1
								and rs.org_id = customer_id_in
								and rs.id = rsc.server_id
								and rsc.channel_id = rcfm.channel_id
								and rcfm.channel_family_id =
									channel_family_id_in
                                -- we only want to grab servers consuming
                                -- physical slots.
                                and exists (
                                    select 1
                                    from rhnChannelFamilyServerPhysical cfsp
                                    where cfsp.server_id = rs.id
                                    and cfsp.channel_family_id =
                                        channel_family_id_in
                                    )
							order by time asc
						)
					)
					where row_number > quantity_in
				);
	begin
		-- if we get a null customer_id, this is completely bogus.
		if customer_id_in is null then
			return;
		end if;

		update		rhnPrivateChannelFamily
			set		max_members = quantity_in
			where	1=1
				and org_id = customer_id_in
				and channel_family_id = channel_family_id_in;

		for sc in serverchannels loop
			rhn_channel.unsubscribe_server(sc.server_id, sc.channel_id, 1, 1);
		end loop;
	end prune_family;

	procedure set_family_count (
		customer_id_in in number,
		channel_family_id_in in number,
		quantity_in in number
	) is
		cursor privperms is
			select	1
			from	rhnPrivateChannelFamily
			where	org_id = customer_id_in
				and channel_family_id = channel_family_id_in;
		cursor pubperms is
			select	o.id org_id
			from	web_customer o,
					rhnPublicChannelFamily pcf
			where	pcf.channel_family_id = channel_family_id_in;
		quantity number;
		done number := 0;
	begin
		quantity := quantity_in;
		if quantity is not null and quantity < 0 then
			quantity := 0;
		end if;

		if customer_id_in is not null then
			for perm in privperms loop
				rhn_entitlements.prune_family(
					customer_id_in,
					channel_family_id_in,
					quantity
				);
				update rhnPrivateChannelFamily
					set max_members = quantity
					where org_id = customer_id_in
						and channel_family_id = channel_family_id_in;
				return;
			end loop;

			insert into rhnPrivateChannelFamily (
					channel_family_id, org_id, max_members, current_members
				) values (
					channel_family_id_in, customer_id_in, quantity, 0
				);
			return;
		end if;

		for perm in pubperms loop
			if quantity = 0 then
				rhn_entitlements.prune_family(
					perm.org_id,
					channel_family_id_in,
					quantity
				);
				if done = 0 then
					delete from rhnPublicChannelFamily
						where channel_family_id = channel_family_id_in;
				end if;
			end if;
			done := 1;
		end loop;
		-- if done's not 1, then we don't have any entitlements
		if done != 1 then
			insert into rhnPublicChannelFamily (
					channel_family_id
				) values (
					channel_family_id_in
				);
		end if;
	end set_family_count;

	-- this expects quantity_in to be the number of available slots, not the
	-- max_members of the server group.  If you give it too many, it'll fail
	-- and raise servergroup_max_members.
	-- We should NEVER run this unless we're SURE that we won't
	-- be violating the max.
	procedure entitle_last_modified_servers (
		customer_id_in in number,
		type_label_in in varchar2,
		quantity_in in number
	) is
		-- find the servers that aren't currently in slots
		cursor servers(cid_in in number, quant_in in number) is
			select	server_id
			from	(
						select	rownum row_number,
								server_id
						from	(
									select	rs.id server_iD
									from	rhnServer rs
									where	1=1
										and rs.org_id = cid_in
										and not exists (
											select	1
											from	rhnServerGroup sg,
													rhnServerGroupMembers rsgm
											where	rsgm.server_id = rs.id
												and rsgm.server_group_id = sg.id
												and sg.group_type is not null
										)
                                        and not exists (
                                            select 1
                                            from rhnVirtualInstance vi
                                            where vi.virtual_system_id =
                                                  rs.id
                                        )                                                                           order by modified desc
								)
					)
			where	row_number <= quant_in;
	begin
		for server in servers(customer_id_in, quantity_in) loop
			rhn_entitlements.entitle_server(server.server_id, type_label_in);
		end loop;
	end entitle_last_modified_servers;

	procedure prune_everything (
		customer_id_in in number
	) is
		cursor everything is
			-- all our server groups
			select	sg.id					id,
					'S'						type,
					sg.max_members			quantity
			from	rhnServerGroup			sg
			where	sg.org_id = customer_id_in
			union
			-- all our user groups
			select	ug.id					id,
					'U'						type,
					ug.max_members 			quantity
			from	rhnUserGroup			ug
			where	ug.org_id = customer_id_in
			union (
			-- all the channel families we have perms to
			select	cfp.channel_family_id	id,
					'C'						type,
					cfp.max_members			quantity
			from	rhnOrgChannelFamilyPermissions cfp
			where	cfp.org_id = customer_id_in
			union
			-- plus all the ones we're using that we have no perms for
			select	cfm.channel_family_id	id,
					'C'						type,
					0						quantity
			from	rhnChannelFamily		cf,
					rhnChannelFamilyMembers	cfm,
					rhnServerChannel		sc,
					rhnServer				s
			where	s.org_id = customer_id_in
				and s.id = sc.server_id
				and sc.channel_id = cfm.channel_id
				and cfm.channel_family_id = cf.id
				and cf.org_id is not null
				and cf.org_id != customer_id_in
				and not exists (
					select	1
					from	rhnOrgChannelFamilyPermissions cfp
					where	cfp.org_id = customer_id_in
						and cfp.channel_family_id = cfm.channel_family_id
					)
			);
	begin
		for one in everything loop
			if one.type in ('U','S') then
				prune_group(one.id, one.type, one.quantity);
			else
				prune_family(customer_id_in, one.id, one.quantity);
			end if;
		end loop;
	end prune_everything;

	procedure subscribe_newest_servers (
		customer_id_in in number
	) is
		-- find servers without base channels
		cursor servers(cid_in in number) is
			select	s.id
			from	rhnServer			s
			where	1=1
				and s.org_id = cid_in
				and not exists (
						select 1
						from	rhnChannel			c,
								rhnServerChannel	sc
						where	sc.server_id = s.id
							and sc.channel_id = c.id
							and c.parent_channel is null
					)
                and not exists (
                        select 1
                        from rhnVirtualInstance vi
                        where vi.virtual_system_id = s.id
                    )
			order by s.modified desc;
		channel_id number;
	begin
		for server in servers(customer_id_in) loop
			channel_id := rhn_channel.guess_server_base(server.id);
			if channel_id is not null then
				begin
					rhn_channel.subscribe_server(server.id, channel_id);
					commit;
				-- exception is really channel_family_no_subscriptions
				exception
					when others then
						null;
				end;
			end if;
		end loop;
	end subscribe_newest_servers;
end rhn_entitlements;
/
show errors


--
-- 20. PRODUCT_NAME_MOD_TRIG  (Trigger)
--
--
-- PRODUCT_NAME_MOD_TRIG  (Trigger)
--
--  Dependencies:
--   RHNPRODUCTNAME (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER PRODUCT_NAME_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNPRODUCTNAME
FOR EACH ROW
begin
    :new.modified := sysdate;
end;
/
SHOW ERRORS;


--
-- 21. PRODUCT_NAME_ID  (Column)
--
ALTER TABLE RHNCHANNEL ADD (PRODUCT_NAME_ID  NUMBER);

--
-- 22. RECEIVING_UPDATES  (Column)
--
 ALTER TABLE RHNCHANNEL ADD (RECEIVING_UPDATES  CHAR(1 BYTE)               DEFAULT 'Y' CONSTRAINT RHN_CHANNEL_RU_NN NOT NULL);

--
-- 23. RHNCHANNELPARENT  (Table)
--
--
-- RHNCHANNELPARENT  (Table)
--
--  Dependencies:
--   RHNCHANNEL (Table)
--
CREATE TABLE RHNCHANNELPARENT
(
  CHANNEL         NUMBER CONSTRAINT RHN_CP_CH_NN NOT NULL,
  PARENT_CHANNEL  NUMBER CONSTRAINT RHN_CP_PARENT_CH_NN NOT NULL,
  CREATED         DATE                          DEFAULT (sysdate) CONSTRAINT RHN_CHP_CREATED_NN NOT NULL,
  MODIFIED        DATE                          DEFAULT (sysdate) CONSTRAINT RHN_CHP_MODIFIED_NN NOT NULL
)
LOGGING
NOCACHE
NOPARALLEL;

--
-- 24. RHN_CP_C_UQ  (Index)
--
--
-- RHN_CP_C_UQ  (Index)
--
--  Dependencies:
--   RHNCHANNELPARENT (Table)
--
CREATE UNIQUE INDEX RHN_CP_C_UQ ON RHNCHANNELPARENT
(CHANNEL, PARENT_CHANNEL)
LOGGING
NOPARALLEL;

--
-- 25. CREATE_FIRST_ORG  (Procedure)
--
--
-- CREATE_FIRST_ORG  (Procedure)
--
--  Dependencies:
--   DUAL ()
--   RHNORGQUOTA (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHN_SERVER_GROUP_ID_SEQ (Sequence)
--   RHN_USER_GROUP_ID_SEQ (Sequence)
--   WEB_CUSTOMER (Table)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure
create_first_org
(
	name_in in varchar2,
	password_in in varchar2
) is
	ug_type			number;
	group_val		number;
begin
	insert into web_customer (
		id, name, password,
		oracle_customer_id, oracle_customer_number,
		customer_type
	) values (
		1, name_in, password_in,
		1, 1, 'B'
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Administrators',
		'Organization Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_applicant';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) VALues (
		group_val, 'Organization Applicants',
		'Organization Applicants for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'system_group_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'System Group Administrators',
		'System Group Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'activation_key_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Activation Key Administrators',
		'Activation Key Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'channel_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Channel Administrators',
		'Channel Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'satellite_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Satellite Administrators',
		'Satellite Administrators for Org ' || name_in || ' (1)',
		NULL, ug_type, 1
	);
	insert into rhnOrgQuota(
		org_id, total
	) values (
		1, 1024*1024*1024*16
	);
        insert into rhnServerGroup
		( id, name, description, max_members, group_type, org_id )
		select rhn_server_group_id_seq.nextval, sgt.name, sgt.name,
			0, sgt.id, 1
		from rhnServerGroupType sgt
		where sgt.label = 'sw_mgr_entitled';
end create_first_org;
/

SHOW ERRORS;

--
-- 26. CREATE_NEW_ORG  (Procedure)
--
--
-- CREATE_NEW_ORG  (Procedure)
--
--  Dependencies:
--   DUAL ()
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHN_SERVER_GROUP_ID_SEQ (Sequence)
--   RHN_USER_GROUP_ID_SEQ (Sequence)
--   WEB_CUSTOMER (Table)
--   WEB_CUSTOMER_ID_SEQ (Sequence)
--   DUAL (Synonym)
--   DUAL (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure
create_new_org
(
	name_in      in varchar2,
	password_in  in varchar2,
	org_id_out   out number
) is
	ug_type			number;
	group_val		number;
	new_org_id              number;
begin
        select web_customer_id_seq.nextval into new_org_id from dual;
	insert into web_customer (
		id, name, password,
		oracle_customer_id, oracle_customer_number,
		customer_type
	) values (
		new_org_id, name_in, password_in,
		new_org_id, new_org_id, 'B'
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Organization Administrators',
		'Organization Administrators for Org ' || name_in,
		NULL, ug_type, new_org_id
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'org_applicant';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) VALues (
		group_val, 'Organization Applicants',
		'Organization Applicants for Org ' || name_in,
		NULL, ug_type, new_org_id
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'system_group_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'System Group Administrators',
		'System Group Administrators for Org ' || name_in,
		NULL, ug_type, new_org_id
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'activation_key_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Activation Key Administrators',
		'Activation Key Administrators for Org ' || name_in,
		NULL, ug_type, new_org_id
	);
	select rhn_user_group_id_seq.nextval into group_val from dual;
	select	id
	into	ug_type
	from	rhnUserGroupType
	where	label = 'channel_admin';
	insert into rhnUserGroup (
		id, name,
		description,
		max_members, group_type, org_id
	) values (
		group_val, 'Channel Administrators',
		'Channel Administrators for Org ' || name_in,
		NULL, ug_type, new_org_id
	);
        insert into rhnServerGroup
		( id, name, description, max_members, group_type, org_id )
		select rhn_server_group_id_seq.nextval, sgt.name, sgt.name,
			0, sgt.id, new_org_id
		from rhnServerGroupType sgt
		where sgt.label = 'sw_mgr_entitled';
	org_id_out := new_org_id;
end create_new_org;
/

SHOW ERRORS;

--
-- 27. RHNVISIBLEOBJECTS  (Table)
--
--
-- RHNVISIBLEOBJECTS  (Table)
--
--  Dependencies:
--   PXTSESSIONS (Table)
--
CREATE TABLE RHNVISIBLEOBJECTS
(
  PXT_SESSION_ID  NUMBER                        NOT NULL,
  OBJECT_ID       NUMBER                        NOT NULL,
  OBJECT_TYPE     VARCHAR2(40 BYTE)             NOT NULL
)
LOGGING
NOCACHE
NOPARALLEL;

--
-- 28. RHN_VIS_OBJS_SESS_OBJ_TYPE_IDX  (Index)
--
--
-- RHN_VIS_OBJS_SESS_OBJ_TYPE_IDX  (Index)
--
--  Dependencies:
--   RHNVISIBLEOBJECTS (Table)
--
CREATE UNIQUE INDEX RHN_VIS_OBJS_SESS_OBJ_TYPE_IDX ON RHNVISIBLEOBJECTS
(PXT_SESSION_ID, OBJECT_ID, OBJECT_TYPE)
LOGGING
NOPARALLEL;

--
-- 29. RHN_SG_DEL_TRIG  (Trigger)
--
--
-- RHN_SG_DEL_TRIG  (Trigger)
--
--  Dependencies:
--   RHNSERVERGROUP (Table)
--   LOOKUP_SNAPSHOT_INVALID_REASON (Function)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTSERVERGROUP (Table)
--   STANDARD (Package)
--
CREATE OR REPLACE TRIGGER RHN_SG_DEL_TRIG
BEFORE DELETE ON RHNSERVERGROUP
FOR EACH ROW
declare
	cursor snapshots is
		select	snapshot_id id
		from	rhnSnapshotServerGroup
		where	server_group_id = :old.id;
begin
	for snapshot in snapshots loop
		update rhnSnapshot
			set invalid = lookup_snapshot_invalid_reason('sg_removed')
			where id = snapshot.id;
		delete from rhnSnapshotServerGroup
			where snapshot_id = snapshot.id
				and server_group_id = :old.id;
	end loop;
end;
/
SHOW ERRORS;

--
--  RHN_SERVER (Package)
--

create or replace
package rhn_server
is

    -- i.e., "can this box do management stuff?" and yes if provisioning box
    function system_service_level(
	server_id_in in number,
	service_level_in in varchar2
    ) return number;

    function can_change_base_channel(
	server_id_in in number
    ) return number;

    procedure set_custom_value(
	server_id_in in number,
	user_id_in in number,
	key_label_in varchar2,
	value_in in varchar2
    );

    function bulk_set_custom_value(
	key_label_in in varchar2,
	value_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) return integer;

    procedure snapshot_server(
	server_id_in in number,
	reason_in in varchar2
    );

    procedure bulk_snapshot(
	reason_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );

    procedure tag_delete(
	server_id_in in number,
	tag_id_in in number
    );

    procedure tag_snapshot(
	snapshot_id_in in number,
	org_id_in in number,
	tagname_in in varchar2
    );

    procedure bulk_snapshot_tag(
	org_id_in in number,
	tagname_in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    );

    procedure remove_action(
	server_id_in in number,
	action_id_in in number
    );

    function check_user_access(server_id_in in number, user_id_in in number) return number;


    function can_server_consume_virt_slot(server_id_in in number,
                                              group_type_in in
                                              rhnServerGroupType.label%TYPE)
    return number;

    procedure insert_into_servergroup (
	server_id_in in number,
	server_group_id_in in number
    );

    function insert_into_servergroup_maybe (
	server_id_in in number,
	server_group_id_in in number
    ) return number;

	procedure insert_set_into_servergroup (
	server_group_id_in in number,
	user_id_in in number,
	set_label_in in varchar2
	);

    procedure delete_from_servergroup (
	server_id_in in number,
	server_group_id_in in number
    );

	procedure delete_set_from_servergroup (
	server_group_id_in in number,
	user_id_in in number,
	set_label_in in varchar2
	);

	procedure clear_servergroup (
	server_group_id_in in number
	);

	procedure delete_from_org_servergroups (
	server_id_in in number
	);

	function get_ip_address (
		server_id_in in number
	) return varchar2;
end rhn_server;
/
SHOW ERRORS


create or replace
package body rhn_server
is
    function system_service_level(
	server_id_in in number,
	service_level_in in varchar2
    ) return number is

    cursor ents is
      select label from rhnServerEntitlementView
      where server_id = server_id_in;

    retval number := 0;

    begin
         for ent in ents loop
            retval := rhn_entitlements.entitlement_grants_service (ent.label, service_level_in);
            if retval = 1 then
               return retval;
            end if;
         end loop;

         return retval;

    end system_service_level;


    function can_change_base_channel(server_id_in IN NUMBER)
    return number
    is
	throwaway number;
    begin
	-- the idea: if we get past this query, the server is
	-- neither sat nor proxy, so base channel is changeable

	select 1 into throwaway
	  from rhnServer S
	 where S.id = server_id_in
	   and not exists (select 1 from rhnSatelliteInfo SI where SI.server_id = S.id)
	   and not exists (select 1 from rhnProxyInfo PI where PI.server_id = S.id);

	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end can_change_base_channel;

    procedure set_custom_value(
	server_id_in in number,
	user_id_in in number,
	key_label_in in varchar2,
	value_in in varchar2
    ) is
	key_id_val number;
    begin
	select CDK.id into key_id_val
	  from rhnCustomDataKey CDK,
	       rhnServer S
	 where S.id = server_id_in
	   and S.org_id = CDK.org_id
	   and CDK.label = key_label_in;

	begin
	    insert into rhnServerCustomDataValue (server_id, key_id, value, created_by, last_modified_by)
	    values (server_id_in, key_id_val, value_in, user_id_in, user_id_in);
	exception
	    when DUP_VAL_ON_INDEX
		then
		update rhnServerCustomDataValue
		   set value = value_in,
		       last_modified_by = user_id_in
		 where server_id = server_id_in
		   and key_id = key_id_val;
	end;

    end set_custom_value;

    function bulk_set_custom_value(
	key_label_in in varchar2,
	value_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    )
    return integer
    is
    i integer := 0;
    begin
        i := 0;
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
		rhn_server.set_custom_value(server.element, set_uid_in, key_label_in, value_in);
            i := i + 1;
	    end if;
	end loop server;
    return i;
    end bulk_set_custom_value;

    procedure bulk_snapshot_tag(
	org_id_in in number,
        tagname_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
	snapshot_id number;
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
		begin
		    select max(id) into snapshot_id
		    from rhnSnapshot
		    where server_id = server.element;
		exception
		    when NO_DATA_FOUND then
			rhn_server.snapshot_server(server.element, 'tagging system:  ' || tagname_in);

			select max(id) into snapshot_id
			from rhnSnapshot
			where server_id = server.element;
		end;

		-- now have a snapshot_id to work with...
		begin
		    rhn_server.tag_snapshot(snapshot_id, org_id_in, tagname_in);
		exception
		    when DUP_VAL_ON_INDEX
			then
			-- do nothing, be forgiving...
			null;
		end;
	    end if;
	end loop server;
    end bulk_snapshot_tag;

    procedure tag_delete(
	server_id_in in number,
	tag_id_in in number
    ) is
	cursor snapshots is
		select	snapshot_id
		from	rhnSnapshotTag
		where	tag_id = tag_id_in;
	tag_id_tmp number;
    begin
	select	id into tag_id_tmp
	from	rhnTag
	where	id = tag_id_in
	for update;

	delete
		from	rhnSnapshotTag
		where	server_id = server_id_in
			and tag_id = tag_id_in;
	for snapshot in snapshots loop
		return;
	end loop;
	delete
		from rhnTag
		where id = tag_id_in;
    end tag_delete;

    procedure tag_snapshot(
        snapshot_id_in in number,
	org_id_in in number,
	tagname_in in varchar2
    ) is
    begin
	insert into rhnSnapshotTag (snapshot_id, server_id, tag_id)
	select snapshot_id_in, server_id, lookup_tag(org_id_in, tagname_in)
	from rhnSnapshot
	where id = snapshot_id_in;
    end tag_snapshot;

    procedure bulk_snapshot(
	reason_in in varchar2,
	set_label_in in varchar2,
	set_uid_in in number
    ) is
    begin
	for server in rhn_set.set_iterator(set_label_in, set_uid_in)
	loop
	    if rhn_server.system_service_level(server.element, 'provisioning') = 1 then
		rhn_server.snapshot_server(server.element, reason_in);
	    end if;
	end loop server;
    end bulk_snapshot;

    procedure snapshot_server(
	server_id_in in number,
	reason_in in varchar2
    ) is
	snapshot_id number;
	cursor revisions is
		select distinct
			cr.id
		from	rhnConfigRevision	cr,
			rhnConfigFileName	cfn,
			rhnConfigFile		cf,
			rhnConfigChannel	cc,
			rhnServerConfigChannel	scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			and cc.id = cf.config_channel_id
			and cf.id = cr.config_file_id
			and cr.id = cf.latest_config_revision_id
			and cf.config_file_name_id = cfn.id
			and cf.id = lookup_first_matching_cf(scc.server_id, cfn.path);
	locked integer;
    begin
	select rhn_snapshot_id_seq.nextval into snapshot_id from dual;

	insert into rhnSnapshot (id, org_id, server_id, reason) (
		select	snapshot_id,
			s.org_id,
			server_id_in,
			reason_in
		from	rhnServer s
		where	s.id = server_id_in
	);
	insert into rhnSnapshotChannel (snapshot_id, channel_id) (
		select	snapshot_id, sc.channel_id
		from	rhnServerChannel sc
		where	sc.server_id = server_id_in
	);
	insert into rhnSnapshotServerGroup (snapshot_id, server_group_id) (
		select	snapshot_id, sgm.server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in
	);
        locked := 0;
        while true loop
            begin
                insert into rhnPackageNEVRA (id, name_id, evr_id, package_arch_id)
                select rhn_pkgnevra_id_seq.nextval, sp.name_id, sp.evr_id, sp.package_arch_id
                from rhnServerPackage sp
                where sp.server_id = server_id_in
                        and not exists
                        (select 1
                                from rhnPackageNEVRA nevra
                                where nevra.name_id = sp.name_id
                                        and nevra.evr_id = sp.evr_id
                                        and (nevra.package_arch_id = sp.package_arch_id
                                            or (nevra.package_arch_id is null
                                                and sp.package_arch_id is null)));
                exit;
            exception when dup_val_on_index then
                if locked = 1 then
                    raise;
                else
                    lock table rhnPackageNEVRA in exclusive mode;
                    locked := 1;
                end if;
            end;
        end loop;
	insert into rhnSnapshotPackage (snapshot_id, nevra_id) (
                select distinct snapshot_id, nevra.id
                from    rhnServerPackage sp, rhnPackageNEVRA nevra
                where   sp.server_id = server_id_in
                        and nevra.name_id = sp.name_id
                        and nevra.evr_id = sp.evr_id
                        and (nevra.package_arch_id = sp.package_arch_id
                            or (nevra.package_arch_id is null
                                and sp.package_arch_id is null))
	);

	insert into rhnSnapshotConfigChannel ( snapshot_id, config_channel_id ) (
		select	snapshot_id, scc.config_channel_id
		from	rhnServerConfigChannel scc
		where	server_id = server_id_in
	);

	for revision in revisions loop
		insert into rhnSnapshotConfigRevision (
				snapshot_id, config_revision_id
			) values (
				snapshot_id, revision.id
			);
	end loop;
    end snapshot_server;

    procedure remove_action(
	server_id_in in number,
	action_id_in in number
    ) is
	-- this really wants "nulls last", but 8.1.7.3.0 sucks ass.
	-- instead, we make a local table that holds our
	-- list of ids with null prereqs.  There's surely a better way
	-- (an array instead of a table maybe?  who knows...)
	-- but I've got code to do this handy that I can look at ;)
	cursor chained_actions is
		select	id, prerequisite
		from	rhnAction
		start with id = action_id_in
		connect by prior id = prerequisite
		order by prerequisite desc;
	cursor sessions is
		select	s.id
		from	rhnKickstartSession s
		where	server_id_in in (s.old_server_id, s.new_server_id)
			and s.action_id = action_id_in
			and not exists (
				select	1
				from	rhnKickstartSessionState ss
				where	ss.id = s.state_id
					and ss.label in ('failed','complete')
			);
	type chain_end_type is table of number index by binary_integer;
	chain_ends chain_end_type;
	i number;
	prereq number := 1;
    begin
	select	prerequisite
	into	prereq
	from	rhnAction
	where	id = action_id_in;

	if prereq is not null then
		rhn_exception.raise_exception('action_is_child');
	end if;

	i := 0;
	for action in chained_actions loop
		if action.prerequisite is null then
			chain_ends(i) := action.id;
			i := i + 1;
		else
			delete from rhnServerAction
				where server_id = server_id_in
				and action_id = action.id;
		end if;
	end loop;
	i := chain_ends.first;
	while i is not null loop
		delete from rhnServerAction
			where server_id = server_id_in
			and action_id = chain_ends(i);
		i := chain_ends.next(i);
	end loop;
	for s in sessions loop
		update rhnKickstartSession
			set 	state_id = (
					select	id
					from	rhnKickstartSessionState
					where	label = 'failed'
				),
				action_id = null
			where	id = s.id;
		set_ks_session_history_message(s.id, 'failed', 'Kickstart cancelled due to action removal');
	end loop;
    end remove_action;

    function check_user_access(server_id_in in number, user_id_in in number)
    return number
    is
	has_access number;
    begin
	-- first check; if this returns no rows, then the server/user are in different orgs, and we bail
        select 1 into has_access
	  from rhnServer S,
	       web_contact wc
	 where wc.org_id = s.org_id
	   and s.id = server_id_in
	   and wc.id = user_id_in;

	-- okay, so they're in the same org.  if we have an org admin, they get a free pass
	if rhn_user.check_role(user_id_in, 'org_admin') = 1
	then
	    return 1;
	end if;

	select 1 into has_access
	  from rhnServerGroupMembers SGM,
	       rhnUserServerGroupPerms USG
	 where SGM.server_group_id = USG.server_group_id
	   and SGM.server_id = server_id_in
	   and USG.user_id = user_id_in
	   and rownum = 1;

	return 1;
    exception
	when no_data_found
	    then
	    return 0;
    end check_user_access;

    -- *******************************************************************
    -- FUNCTION: can_server_consume_virt_slot
    -- Returns 1 if the server id is eligible to consume a virtual slot,
    --   else returns 0.
    -- Called by: insert_into_servergroup, delete_from_servergroup
    -- *******************************************************************
    function can_server_consume_virt_slot(server_id_in in number,
                                           group_type_in in
                                           rhnServerGroupType.label%TYPE)
    return number
    is

        cursor server_virt_slots is
            select vi.VIRTUAL_SYSTEM_ID
            from
                rhnVirtualInstance vi
            where
                -- server id is a virtual instance
                vi.VIRTUAL_SYSTEM_ID = server_id_in
                -- server id's host is virt entitled
                and exists ( select 1
                     from rhnServerEntitlementView sev
                 where vi.HOST_SYSTEM_ID = sev.server_id
                 and sev.label in ('virtualization_host',
                                   'virtualization_host_platform') )
                -- server id's host also has the ent we want
                and exists ( select 1
                     from rhnServerEntitlementView sev2
                 where vi.HOST_SYSTEM_ID = sev2.server_id
                 and sev2.label = group_type_in );

    begin
        for server_virt_slot in server_virt_slots loop
            return 1;
        end loop;
        return 0;
    end can_server_consume_virt_slot;


    procedure insert_into_servergroup (
		server_id_in in number,
		server_group_id_in in number
    ) is
		cursor sg_users is
			select	user_id id
			from	rhnUserServerGroupPerms
			where	server_group_id = server_group_id_in;
		used_slots number;
		max_slots number;
		org_id number;
		mgmt_available number;
		mgmt_upgrade number;
		mgmt_sgid number;
		prov_available number;
		prov_upgrade number;
		prov_sgid number;
		group_label rhnServerGroupType.label%TYPE;
		group_type number;
	begin
		-- frist, group_type = null, because it's easy...

		-- this will rowlock the servergroup we're trying to change;
		-- we probably need to lock the other one, but I think the chances
		-- of it being a real issue are very small for now...
		select	sg.group_type, sg.org_id, sg.current_members, sg.max_members
		into	group_type, org_id, used_slots, max_slots
		from	rhnServerGroup sg
		where	sg.id = server_group_id_in
		for update of sg.current_members;

		if group_type is null then
			if used_slots >= max_slots then
				rhn_exception.raise_exception('servergroup_max_members');
			end if;

			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);
			update rhnServerGroup
				set current_members = current_members + 1
				where id = server_group_id_in;

			for u in sg_users loop
				rhn_cache.update_perms_for_user(u.id);
			end loop;
			return;
		end if;

		-- now for group_type != null
		--
		select	label
		into	group_label
		from	rhnServerGroupType	sgt
		where	sgt.id = group_type;

		-- the naive easy path that gets hit most often and has to be quickest.
		if group_label in ('sw_mgr_entitled',
                           'enterprise_entitled',
                           'monitoring_entitled',
                           'provisioning_entitled',
                           'virtualization_host',
                           'virtualization_host_platform') then
			if used_slots >= max_slots and
               (can_server_consume_virt_slot(server_id_in, group_label) != 1)
               then
				rhn_exception.raise_exception('servergroup_max_members');
			end if;

			insert into rhnServerGroupMembers(
					server_id, server_group_id
				) values (
					server_id_in, server_group_id_in
				);

            -- Only update current members if the system in consuming a
            -- physical slot.
            if can_server_consume_virt_slot(server_id_in, group_label) = 0 then
                update rhnServerGroup
                set current_members = current_members + 1
                where id = server_group_id_in;
            end if;

			return;
		end if;
	end;

	function insert_into_servergroup_maybe (
		server_id_in in number,
		server_group_id_in in number
	) return number is
		retval number := 0;
		cursor servergroups is
			select	s.id	server_id,
					sg.id	server_group_id
			from	rhnServerGroup	sg,
					rhnServer		s
			where	s.id = server_id_in
				and sg.id = server_group_id_in
				and s.org_id = sg.org_id
				and not exists (
					select	1
					from	rhnServerGroupMembers sgm
					where	sgm.server_id = s.id
						and sgm.server_group_id = sg.id
				);
	begin
		for sgm in servergroups loop
			rhn_server.insert_into_servergroup(sgm.server_id, sgm.server_group_id);
			retval := retval + 1;
		end loop;
		return retval;
	end insert_into_servergroup_maybe;

	procedure insert_set_into_servergroup (
		server_group_id_in in number,
		user_id_in in number,
		set_label_in in varchar2
	) is
		cursor servers is
			select	st.element	id
			from	rhnSet		st
			where	st.user_id = user_id_in
				and st.label = set_label_in
				and exists (
					select	1
					from	rhnUserManagedServerGroups umsg
					where	umsg.server_group_id = server_group_id_in
						and umsg.user_id = user_id_in
					)
				and not exists (
					select	1
					from	rhnServerGroupMembers sgm
					where	sgm.server_id = st.element
						and sgm.server_group_id = server_group_id_in
				);
	begin
		for s in servers loop
			rhn_server.insert_into_servergroup(s.id, server_group_id_in);
		end loop;
	end insert_set_into_servergroup;

    procedure delete_from_servergroup (
	server_id_in in number,
		server_group_id_in in number
    ) is
		cursor sg_users is
			select	user_id id
			from	rhnUserServerGroupPerms
			where	server_group_id = server_group_id_in;

        cursor server_virt_groups is
            select 1
            from rhnServerEntitlementVirtual sev
            where sev.server_id = server_id_in
            and sev.server_group_id = server_group_id_in;

		oid number;
		mgmt_sgid number;
		label rhnServerGroupType.label%TYPE;
		group_type number;
	begin
		begin
			select	sg.group_type, sg.org_id
			into	group_type,	oid
			from	rhnServerGroupMembers	sgm,
					rhnServerGroup			sg
			where	sg.id = server_group_id_in
				and sg.id = sgm.server_group_id
				and sgm.server_id = server_id_in
			for update of sg.current_members;
		exception
			when no_data_found then
				rhn_exception.raise_exception('server_not_in_group');
		end;

		-- do group_type is null first
		if group_type is null then
			delete from rhnServerGroupMembers
				where server_group_id = server_group_id_in
				and	server_id = server_id_in;
			update rhnServerGroup
				set current_members = current_members - 1
				where id = server_group_id_in;
			for u in sg_users loop
				rhn_cache.update_perms_for_user(u.id);
			end loop;
			return;
		end if;

		select	sgt.label
		into	label
		from	rhnServerGroupType sgt
		where	sgt.id = group_type;

		if label in ('sw_mgr_entitled',
                     'enterprise_entitled',
                     'provisioning_entitled',
                     'monitoring_entitled',
                     'virtualization_host',
                     'virtualization_host_platform') then

            -- Only update current members if the system is consuming
            -- a physical slot.
            for server_virt_group in server_virt_groups loop
                delete from rhnServerGroupMembers
                where server_group_id = server_group_id_in
                and	server_id = server_id_in;
                return;
            end loop;

            delete from rhnServerGroupMembers
            where server_group_id = server_group_id_in
            and	server_id = server_id_in;

            update rhnServerGroup
            set current_members = current_members - 1
            where id = server_group_id_in;

		end if;
	end;

	procedure delete_set_from_servergroup (
		server_group_id_in in number,
		user_id_in in number,
		set_label_in in varchar2
	) is
		cursor servergroups is
			select	sgm.server_id, sgm.server_group_id
			from	rhnSet st,
					rhnServerGroupMembers sgm
			where	sgm.server_group_id = server_group_id_in
				and st.user_id = user_id_in
				and st.label = set_label_in
				and sgm.server_id = st.element
				and exists (
					select	1
					from	rhnUserManagedServerGroups usgp
					where	usgp.server_group_id = server_group_id_in
						and usgp.user_id = user_id_in
				);
	begin
		for sgm in servergroups loop
			rhn_server.delete_from_servergroup(sgm.server_id, server_group_id_in);
		end loop;
	end delete_set_from_servergroup;

	procedure clear_servergroup (
		server_group_id_in in number
	) is
		cursor servers is
			select	sgm.server_id	id
			from	rhnServerGroupMembers sgm
			where	sgm.server_group_id = server_group_id_in;
	begin
		for s in servers loop
			rhn_server.delete_from_servergroup(s.id, server_group_id_in);
		end loop;
	end clear_servergroup;

	procedure delete_from_org_servergroups (
		server_id_in in number
	) is
		cursor servergroups is
			select	sgm.server_group_id id
			from	rhnServerGroup sg,
					rhnServerGroupMembers sgm
			where	sgm.server_id = server_id_in
				and sgm.server_group_id = sg.id
				and sg.group_type is null;
	begin
		for sg in servergroups loop
			rhn_server.delete_from_servergroup(server_id_in, sg.id);
		end loop;
	end delete_from_org_servergroups;

	function get_ip_address (
		server_id_in in number
	) return varchar2 is
		cursor interfaces is
			select	name, ip_addr
			from	rhnServerNetInterface
			where	server_id = server_id_in
				and ip_addr != '127.0.0.1';
		cursor addresses is
			select	ipaddr ip_addr
			from	rhnServerNetwork
			where	server_id = server_id_in
				and ipaddr != '127.0.0.1';
	begin
		for addr in addresses loop
			return addr.ip_addr;
		end loop;
		for iface in interfaces loop
			return iface.ip_addr;
		end loop;
		return NULL;
	end get_ip_address;
end rhn_server;
/
SHOW ERRORS

--
-- 30. RHN_ENTITLEMENTS  (Package)
--
--
-- RHN_ENTITLEMENTS  (Package)
--
--  Dependencies:
--   DUAL ()
--   LOOKUP_SG_TYPE (Function)
--   PLITBLM ()
--   RHNCHANNEL (Table)
--   RHNCHANNELFAMILY (Table)
--   RHNCHANNELFAMILYMEMBERS (Table)
--   RHNCHANNELFAMILYSERVERPHYSICAL (View)
--   RHNCHANNELFAMILYSERVERS (View)
--   RHNORGCHANNELFAMILYPERMISSIONS (View)
--   RHNORGENTITLEMENTS (Table)
--   RHNORGENTITLEMENTTYPE (Table)
--   RHNPRIVATECHANNELFAMILY (Table)
--   RHNPUBLICCHANNELFAMILY (Table)
--   RHNSERVER (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERENTITLEMENTPHYSICAL (View)
--   RHNSERVERENTITLEMENTVIEW (View)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNSERVERHISTORY (Table)
--   RHNSERVERSERVERGROUPARCHCOMPAT (Table)
--   RHNSGTYPEBASEADDONCOMPAT (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHNVIRTUALINSTANCE (Table)
--   RHN_CHANNEL (Package)
--   RHN_CHECK_PROBE (Table)
--   RHN_EVENT_ID_SEQ (Sequence)
--   RHN_EXCEPTION (Package)
--   RHN_PROBE (Table)
--   RHN_SERVER (Package)
--   RHN_SERVER_GROUP_ID_SEQ (Sequence)
--   RHN_USER (Package)
--   RHN_USER_GROUP_ID_SEQ (Sequence)
--   STATE_CHANGE (Table)
--   TIME_SERIES (Table)
--   WEB_CUSTOMER (Table)
--   DUAL (Synonym)
--   PLITBLM (Synonym)
--   DUAL (Table)
--   PLITBLM (Package)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE package rhn_entitlements
is
	body_version varchar2(100) := '$Id: rhn_entitlements.pks 131502 2007-09-10 14:20:23Z jslagle $';
   type ents_array is varray(10) of rhnServerGroupType.label%TYPE;
    procedure remove_org_entitlements (
        org_id_in number
    );
    function entitlement_grants_service (
	    entitlement_in in varchar2,
		service_level_in in varchar2
	) return number;
	function lookup_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number;
	function create_entitlement_group (
		org_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	) return number;
   function can_entitle_server (
      server_id_in   in number,
      type_label_in  in varchar2
   )
   return number;
   function can_switch_base (
      server_id_in   in    integer,
      type_label_in  in    varchar2
   )
   return number;
   function find_compatible_sg (
      server_id_in in number,
      type_label_in in varchar2,
      sgid_out out number
   )
   return boolean;
	procedure entitle_server (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled'
	);
	procedure remove_server_entitlement (
		server_id_in in number,
		type_label_in in varchar2 := 'sw_mgr_entitled',
        repoll_virt_guests in number := 1
	);
	procedure unentitle_server (
		server_id_in in number
	);
    procedure repoll_virt_guest_entitlements(
        server_id_in in number
    );
	function get_server_entitlement (
		server_id_in in number
	) return ents_array;
	procedure modify_org_service (
		org_id_in in number,
		service_label_in in varchar2,
		enable_in in char
	);
    procedure set_customer_enterprise (
		customer_id_in in number
	);
	procedure set_customer_provisioning (
		customer_id_in in number
	);
	procedure set_customer_nonlinux (
		customer_id_in in number
	);
    procedure unset_customer_enterprise (
		customer_id_in in number
	);
	procedure unset_customer_provisioning (
		customer_id_in in number
	);
	procedure unset_customer_nonlinux (
		customer_id_in in number
	);
    procedure assign_system_entitlement(
        group_label_in in varchar2,
        from_org_id_in in number,
        to_org_id_in in number,
        quantity_in in number
    );
    procedure assign_channel_entitlement(
        channel_family_label_in in varchar2,
        from_org_id_in in number,
        to_org_id_in in number,
        quantity_in in number
    );
    procedure activate_system_entitlement(
        org_id_in in number,
        group_label_in in varchar2,
        quantity_in in number
    );
    procedure activate_channel_entitlement(
        org_id_in in number,
        channel_family_label_in in varchar2,
        quantity_in in number
    );
    procedure set_group_count (
		customer_id_in in number,	-- customer_id
		type_in in char,			-- 'U' or 'S'
		group_type_in in number,	-- rhn[User|Server]GroupType.id
		quantity_in in number		-- quantity
    );
    procedure set_family_count (
		customer_id_in in number,		-- customer_id
		channel_family_id_in in number,	-- 246
		quantity_in in number			-- 3
    );
    procedure entitle_last_modified_servers (
		customer_id_in in number,	-- customer_id
		type_label_in in varchar2,	-- 'enterprise_entitled'
		quantity_in in number		-- 3
    );
	procedure prune_everything (
		customer_id_in in number
	);
	procedure subscribe_newest_servers (
		customer_id_in in number
	);
end rhn_entitlements;
/

SHOW ERRORS;

--
-- 31. RHN_ORG  (Package)
--
--
-- RHN_ORG  (Package)
--
--  Dependencies:
--   DELETE_SERVER (Procedure)
--   RHNCHANNEL (Table)
--   RHNCONFIGCHANNEL (Table)
--   RHNDAILYSUMMARYQUEUE (Table)
--   RHNFILELIST (Table)
--   RHNMESSAGE (Table)
--   RHNORGINFO (Table)
--   RHNORGQUOTA (Table)
--   RHNREGTOKEN (Table)
--   RHNSERVER (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUP (Table)
--   RHNSERVERGROUPTYPE (Table)
--   RHNUSERGROUP (Table)
--   RHNUSERGROUPMEMBERS (Table)
--   RHNUSERGROUPTYPE (Table)
--   RHNUSERMESSAGE (Table)
--   RHNUSERSERVERGROUPPERMS (Table)
--   RHNUSERSERVERPERMS (Table)
--   RHN_CHECK_SUITES (Table)
--   RHN_COMMAND_QUEUE_SESSIONS (Table)
--   RHN_COMMAND_TARGET (Table)
--   RHN_CONFIG (Package)
--   RHN_CONTACT_GROUPS (Table)
--   RHN_CONTACT_METHODS (Table)
--   RHN_ENTITLEMENTS (Package)
--   RHN_EXCEPTION (Package)
--   RHN_NOTIFICATION_FORMATS (Table)
--   RHN_PROBE (Table)
--   RHN_REDIRECTS (Table)
--   RHN_SAT_CLUSTER (Table)
--   RHN_SCHEDULES (Table)
--   RHN_USER (Package)
--   WEB_CONTACT (Table)
--   WEB_CUSTOMER (Table)
--   STANDARD (Package)
--   STANDARD (Package)
--
CREATE OR REPLACE PACKAGE rhn_org
IS
	version varchar2(100) := '$Id: rhn_org.pks 132550 2007-10-05 16:02:24Z jslagle $';
    CURSOR server_group_by_label(org_id_in NUMBER, group_label_in VARCHAR2) IS
	   SELECT SG.*
	     FROM rhnServerGroupType SGT,
		  rhnServerGroup SG
	    WHERE SG.group_type = SGT.id
	      AND SGT.label = group_label_in
	      AND SG.org_id = org_id_in;
    FUNCTION find_server_group_by_type(org_id_in NUMBER,
                                       group_label_in VARCHAR2)
    RETURN NUMBER;
    procedure delete_org(org_id_in in number);
    procedure delete_user(user_id_in in number, deleting_org in number := 0);
END rhn_org;
/

SHOW ERRORS;

--
-- 19. RHN_ORG  (Package Body)
--
--
-- RHN_ORG  (Package Body)
--

CREATE OR REPLACE PACKAGE BODY rhn_org
IS
	body_version varchar2(100) := '$Id: rhn_org.pkb 132550 2007-10-05 16:02:24Z jslagle $';
    FUNCTION find_server_group_by_type(org_id_in NUMBER, group_label_in VARCHAR2)
    RETURN NUMBER
    IS
	server_group       server_group_by_label%ROWTYPE;
    BEGIN
	OPEN server_group_by_label(org_id_in, group_label_in);
	FETCH server_group_by_label INTO server_group;
	CLOSE server_group_by_label;
	RETURN server_group.id;
    END find_server_group_by_type;
    procedure delete_org (
        org_id_in in number
    )
    is
        cursor users is
        select id
        from web_contact
        where org_id = org_id_in;
		cursor servers(org_id_in in number) is
        select	id
        from	rhnServer
        where	org_id = org_id_in;
        cursor config_channels is
        select id
        from rhnConfigChannel
        where org_id = org_id_in;
    begin
        if org_id_in = 1 then
            rhn_exception.raise_exception('cannot_delete_base_org');
        end if;
        for u in users loop
            rhn_org.delete_user(u.id, 1);
        end loop;
        for s in servers(org_id_in) loop
            delete_server(s.id);
        end loop;
        for c in config_channels loop
            rhn_config.delete_channel(c.id);
        end loop;
        rhn_entitlements.remove_org_entitlements(org_id_in);
        delete from rhnChannel where org_id = org_id_in;
        delete from rhnDailySummaryQueue where org_id = org_id_in;
        delete from rhnOrgQuota where org_id = org_id_in;
        delete from rhnOrgInfo where org_id = org_id_in;
        delete from rhnFileList where org_id = org_id_in;
        delete from rhnServerGroup where org_id = org_id_in;
        delete from rhn_check_suites where customer_id = org_id_in;
        delete from rhn_command_target where customer_id = org_id_in;
        delete from rhn_contact_groups where customer_id = org_id_in;
        delete from rhn_notification_formats where customer_id = org_id_in;
        delete from rhn_probe where customer_id = org_id_in;
        delete from rhn_redirects where customer_id = org_id_in;
        delete from rhn_sat_cluster where customer_id = org_id_in;
        delete from rhn_schedules where customer_id = org_id_in;
        delete from web_customer where id = org_id_in;
    end delete_org;
	procedure delete_user(user_id_in in number, deleting_org in number := 0) is
		cursor is_admin is
			select	1
			from	rhnUserGroupType	ugt,
					rhnUserGroup		ug,
					rhnUserGroupMembers	ugm
			where	ugm.user_id = user_id_in
				and ugm.user_group_id = ug.id
				and ug.group_type = ugt.id
				and ugt.label = 'org_admin';
		cursor servergroups_needing_admins is
			select	usgp.server_group_id	server_group_id
			from	rhnUserServerGroupPerms	usgp
			where	1=1
				and usgp.user_id = user_id_in
				and not exists (
					select	1
					from	rhnUserServerGroupPerms	sq_usgp
					where	1=1
						and sq_usgp.server_group_id = usgp.server_group_id
						and	sq_usgp.user_id != user_id_in
				);
		cursor messages is
			select	message_id id
			from	rhnUserMessage
			where	user_id = user_id_in;
		users			number;
		our_org_id		number;
		other_users		number;
		other_org_admin	number;
        other_user_id  number;
	begin
		select	wc.org_id
		into	our_org_id
		from	web_contact wc
		where	id = user_id_in;
		begin
			select	id, 1
			into	other_user_id, other_users
			from	web_contact
			where	1=1
				and org_id = our_org_id
				and id != user_id_in
				and rownum = 1;
		exception
			when no_data_found then
				other_users := 0;
		end;
		if other_users != 0 then
			for ignore in is_admin loop
				begin
					select	new_ugm.user_id
					into	other_org_admin
					from	rhnUserGroupMembers	new_ugm,
							rhnUserGroupType	ugt,
							rhnUserGroup		ug,
							rhnUserGroupMembers	ugm
					where	ugm.user_id = user_id_in
						and ugm.user_group_id = ug.id
						and ug.group_type = ugt.id
						and ugt.label = 'org_admin'
						and ug.id = new_ugm.user_group_id
						and new_ugm.user_id != user_id_in
						and rownum = 1;
				exception
					when no_data_found then
                        if deleting_org = 0 then
						rhn_exception.raise_exception('cannot_delete_user');
                        end if;
				end;
				for sg in servergroups_needing_admins loop
					rhn_user.add_servergroup_perm(other_org_admin,
						sg.server_group_id);
				end loop;
			end loop;
		end if;
		for message in messages loop
			delete
				from	rhnUserMessage
				where	user_id = user_id_in
					and message_id = message.id;
			begin
				select	1
				into	users
				from	rhnUserMessage
				where	message_id = message.id
					and rownum = 1;
				delete
					from	rhnMessage
					where	id = message.id;
			exception
				when no_data_found then
					null;
			end;
		end loop;
		delete from rhn_command_queue_sessions where contact_id = user_id_in;
		delete from rhn_contact_methods where contact_id = user_id_in;
		delete from rhn_redirects where contact_id = user_id_in;
		delete from rhnUserServerPerms where user_id = user_id_in;
		if other_users != 0 then
			update		rhnRegToken
				set		user_id = nvl(other_org_admin, other_user_id)
				where	org_id = our_org_id
					and user_id = user_id_in;
			begin
				delete from web_contact where id = user_id_in;
			exception
				when others then
					rhn_exception.raise_exception('cannot_delete_user');
			end;
		else
            begin
                delete from web_contact where id = user_id_in;
		    exception
				when others then
					rhn_exception.raise_exception('cannot_delete_user');
			end;
		end if;
		return;
	end delete_user;
END rhn_org;
/

SHOW ERRORS;

--
-- 32. DELETE_SERVER  (Procedure)
--
--
-- DELETE_SERVER  (Procedure)
--
--  Dependencies:
--   RHNACTIONCONFIGCHANNEL (Table)
--   RHNACTIONCONFIGREVISION (Table)
--   RHNACTIONPACKAGEREMOVALFAILURE (Table)
--   RHNAPPINSTALLSESSION (Table)
--   RHNCHANNELFAMILYLICENSECONSENT (Table)
--   RHNCLIENTCAPABILITY (Table)
--   RHNCONFIGCHANNEL (Table)
--   RHNCONFIGCHANNELTYPE (Table)
--   RHNCPU (Table)
--   RHNDEVICE (Table)
--   RHNFILELIST (Table)
--   RHNKICKSTARTPRESERVEFILELIST (Table)
--   RHNKICKSTARTSESSION (Table)
--   RHNPROXYINFO (Table)
--   RHNRAM (Table)
--   RHNREGTOKEN (Table)
--   RHNSATELLITECHANNELFAMILY (Table)
--   RHNSATELLITEINFO (Table)
--   RHNSERVER (Table)
--   RHNSERVERACTION (Table)
--   RHNSERVERACTIONPACKAGERESULT (Table)
--   RHNSERVERACTIONSCRIPTRESULT (Table)
--   RHNSERVERACTIONVERIFYMISSING (Table)
--   RHNSERVERACTIONVERIFYRESULT (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNSERVERCONFIGCHANNEL (Table)
--   RHNSERVERCUSTOMDATAVALUE (Table)
--   RHNSERVERDMI (Table)
--   RHNSERVERENTITLEMENTVIEW (View)
--   RHNSERVEREVENT (Table)
--   RHNSERVERGROUPMEMBERS (Table)
--   RHNSERVERHISTORY (Table)
--   RHNSERVERINFO (Table)
--   RHNSERVERINSTALLINFO (Table)
--   RHNSERVERLOCATION (Table)
--   RHNSERVERLOCK (Table)
--   RHNSERVERMESSAGE (Table)
--   RHNSERVERNEEDEDERRATACACHE (Table)
--   RHNSERVERNEEDEDPACKAGECACHE (Table)
--   RHNSERVERNETINTERFACE (Table)
--   RHNSERVERNETWORK (Table)
--   RHNSERVERNOTES (Table)
--   RHNSERVERPACKAGE (Table)
--   RHNSERVERPATH (Table)
--   RHNSERVERPRESERVEFILELIST (Table)
--   RHNSERVERTOKENREGS (Table)
--   RHNSERVERUUID (Table)
--   RHNSET (Table)
--   RHNSNAPSHOT (Table)
--   RHNSNAPSHOTTAG (Table)
--   RHNSNPSERVERQUEUE (Table)
--   RHNTRANSACTION (Table)
--   RHNUSERSERVERPERMS (Table)
--   RHNUSERSERVERPREFS (Table)
--   RHNVIRTUALINSTANCE (Table)
--   RHNVIRTUALINSTANCEEVENTLOG (Table)
--   RHN_CHANNEL (Package)
--   RHN_CHECK_PROBE (Table)
--   RHN_CONFIG (Package)
--   RHN_ENTITLEMENTS (Package)
--   RHN_HOST_PROBE (Table)
--   RHN_INTERFACE_MONITORING (Table)
--   RHN_PROBE (Table)
--   RHN_PROBE_STATE (Table)
--   RHN_SAT_CLUSTER (Table)
--   RHN_SAT_NODE (Table)
--   RHN_SERVER (Package)
--   RHN_SERVER_MONITORING_INFO (Table)
--   WEB_CONTACT (Table)
--   STANDARD (Package)
--   SYS_STUB_FOR_PURITY_ANALYSIS (Package)
--
CREATE OR REPLACE procedure delete_server (
	server_id_in in number
) is
	cursor servergroups is
		select	server_id, server_group_id
		from	rhnServerGroupMembers sgm
		where	sgm.server_id = server_id_in;
	cursor configchannels is
		select	cc.id
		from	rhnConfigChannel cc,
			rhnConfigChannelType cct,
			rhnServerConfigChannel scc
		where	1=1
			and scc.server_id = server_id_in
			and scc.config_channel_id = cc.id
			and cct.label in
				('local_override','server_import')
			and cct.id = cc.confchan_type_id;
	cursor filelists is
		select	spfl.file_list_id id
		from	rhnServerPreserveFileList spfl
		where	spfl.server_id = server_id_in
			and not exists (
				select	1
				from	rhnServerPreserveFileList
				where	file_list_id = spfl.file_list_id
					and server_id != server_id_in
				union
				select	1
				from	rhnKickstartPreserveFileList
				where	file_list_id = spfl.file_list_id
			);
	cluster_id number;
    is_virt number := 0;
begin
	rhn_channel.delete_server_channels(server_id_in);
	for filelist in filelists loop
		delete from rhnFileList where id = filelist.id;
	end loop;
	for configchannel in configchannels loop
		rhn_config.delete_channel(configchannel.id);
	end loop;
    begin
      select unique 1 into is_virt
        from rhnServerEntitlementView
       where server_id = server_id_in
         and label in ('virtualization_host', 'virtualization_host_platform');
    exception
      when no_data_found then
        is_virt := 0;
    end;
	for sgm in servergroups loop
		rhn_server.delete_from_servergroup(
			sgm.server_id, sgm.server_group_id);
	end loop;
    if is_virt = 1 then
        rhn_entitlements.repoll_virt_guest_entitlements(server_id_in);
    end if;
	update rhnKickstartSession
		set old_server_id = null
		where old_server_id = server_id_in;
	update rhnKickstartSession
		set new_server_id = null
		where new_server_id = server_id_in;
	rhn_channel.clear_subscriptions(server_id_in,1);
        delete from rhnVirtualInstance
	      where host_system_id = server_id_in
		and virtual_system_id is null;
        delete from rhnVirtualInstance
	      where virtual_system_id = server_id_in
	        and host_system_id is null;
    delete from rhnVirtualInstance vi
    where vi.virtual_system_id = server_id_in
    and vi.modified < (select max(vi2.modified)
                    from rhnVirtualInstance vi2
                    where vi2.uuid = vi.uuid);
        update rhnVirtualInstance
	   set host_system_id = null
	 where host_system_id = server_id_in;
	update rhnVirtualInstance
	   set virtual_system_id = null
	 where virtual_system_id = server_id_in;
	update rhnVirtualInstanceEventLog
	   set old_host_system_id = null
         where old_host_system_id = server_id_in;
	update rhnVirtualInstanceEventLog
	   set new_host_system_id = null
         where new_host_system_id = server_id_in;
	delete from rhnActionConfigChannel where server_id = server_id_in;
	delete from rhnActionConfigRevision where server_id = server_id_in;
	delete from rhnActionPackageRemovalFailure where server_id = server_id_in;
	delete from rhnChannelFamilyLicenseConsent where server_id = server_id_in;
	delete from rhnClientCapability where server_id = server_id_in;
	delete from rhnCpu where server_id = server_id_in;
	delete from rhnDevice where server_id = server_id_in;
	delete from rhnProxyInfo where server_id = server_id_in;
	delete from rhnRam where server_id = server_id_in;
	delete from rhnRegToken where server_id = server_id_in;
	delete from rhnSNPServerQueue where server_id = server_id_in;
	delete from rhnSatelliteChannelFamily where server_id = server_id_in;
	delete from rhnSatelliteInfo where server_id = server_id_in;
	delete from rhnServerAction where server_id = server_id_in;
	delete from rhnServerActionPackageResult where server_id = server_id_in;
	delete from rhnServerActionScriptResult where server_id = server_id_in;
	delete from rhnServerActionVerifyResult where server_id = server_id_in;
	delete from rhnServerActionVerifyMissing where server_id = server_id_in;
	delete from rhnServerChannel where server_id = server_id_in;
	delete from rhnServerConfigChannel where server_id = server_id_in;
	delete from rhnServerCustomDataValue where server_id = server_id_in;
	delete from rhnServerDMI where server_id = server_id_in;
	delete from rhnServerMessage where server_id = server_id_in;
	delete from rhnServerEvent where server_id = server_id_in;
	delete from rhnServerHistory where server_id = server_id_in;
	delete from rhnServerInfo where server_id = server_id_in;
	delete from rhnServerInstallInfo where server_id = server_id_in;
	delete from rhnServerLocation where server_id = server_id_in;
	delete from rhnServerLock where server_id = server_id_in;
	delete from rhnServerNeededPackageCache where server_id = server_id_in;
	delete from rhnServerNeededErrataCache where server_id = server_id_in;
	delete from rhnServerNetwork where server_id = server_id_in;
	delete from rhnServerNotes where server_id = server_id_in;
	delete from rhnServerPackage where server_id = server_id_in;
	delete from rhnServerTokenRegs where server_id = server_id_in;
	delete from rhnSnapshotTag where server_id = server_id_in;
	delete from rhnSnapshot where server_id = server_id_in;
	delete from rhnTransaction where server_id = server_id_in;
	delete from rhnUserServerPrefs where server_id = server_id_in;
	delete from rhnServerPath where server_id_in in (server_id, proxy_server_id);
	delete from rhnUserServerPerms where server_id = server_id_in;
	delete from rhn_interface_monitoring where server_id = server_id_in;
	delete from rhnServerNetInterface where server_id = server_id_in;
	delete from rhn_server_monitoring_info where recid = server_id_in;
	delete from rhnAppInstallSession where server_id = server_id_in;
	delete from rhnServerUuid where server_id = server_id_in;
    DELETE FROM rhn_probe_state PS WHERE PS.probe_id in
    (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.host_id = server_id_in
    );
    DELETE FROM rhn_probe P  WHERE P.recid in
    (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.host_id = server_id_in
    );
    DELETE FROM rhn_probe_state PS
      WHERE PS.probe_id in
      (SELECT CP.probe_id
       FROM rhn_check_probe CP
      WHERE CP.sat_cluster_id in
    (SELECT SN.sat_cluster_id
       FROM rhn_sat_node SN
      WHERE SN.server_id = server_id_in));
    DELETE FROM rhn_probe P
       WHERE P.recid in
      (SELECT CP.probe_id
         FROM rhn_check_probe CP
         WHERE CP.sat_cluster_id in
      (SELECT SN.sat_cluster_id
         FROM rhn_sat_node SN
         WHERE SN.server_id = server_id_in));
	delete from rhn_check_probe where host_id = server_id_in;
	delete from rhn_host_probe where host_id = server_id_in;
    delete from rhn_sat_cluster where recid in
      ( select sat_cluster_id from rhn_sat_node where server_id = server_id_in );
	delete from rhn_sat_node where server_id = server_id_in;
	delete
	from	rhnServer
		where id = server_id_in;
	delete
	from	rhnSet
	where	1=1
		and user_id in (
			select	wc.id
			from	rhnServer rs,
				web_contact wc
			where	rs.id = server_id_in
				and rs.org_id = wc.org_id
		)
		and label = 'system_list'
		and element = server_id_in;
end delete_server;
/

SHOW ERRORS;

--
-- 33. RHNUSERAVAILABLECHANNELS  (View)
--
--
-- RHNUSERAVAILABLECHANNELS  (View)
--
--  Dependencies:
--   RHNCHANNELPERMISSION (Table)
--   RHNSERVERCHANNEL (Table)
--   RHNUSERCHANNELTREEVIEW (View)
--   RHNUSERSERVERPERMS (Table)
--   RHN_CHANNEL (Package)
--
CREATE OR REPLACE FORCE VIEW RHNUSERAVAILABLECHANNELS
(USER_ID, ORG_ID, CHANNEL_ID, CHANNEL_DEPTH, CHANNEL_NAME,
 CHANNEL_ARCH_ID, PADDED_NAME, CURRENT_MEMBERS, AVAILABLE_MEMBERS, LAST_MODIFIED,
 CHANNEL_LABEL, PARENT_OR_SELF_LABEL, PARENT_OR_SELF_ID, END_OF_LIFE)
AS
select
     ct.user_id,
     ct.org_id,
     ct.id,
     CT.depth,
     CT.name,
     CT.channel_arch_id,
     CT.padded_name,
     (
     SELECT COUNT(1)
       FROM rhnUserServerPerms USP
      WHERE USP.user_id = ct.user_id
        AND EXISTS (SELECT 1 FROM rhnServerChannel WHERE channel_id = ct.id AND server_id = USP.server_id)
     ),
     rhn_channel.available_chan_subscriptions(ct.id, ct.org_id),
     CT.last_modified,
     CT.label,
     CT.parent_or_self_label,
     CT.parent_or_self_id,
     CT.end_of_life
from
     rhnUserChannelTreeView ct
where rhn_channel.org_channel_setting(ct.id, ct.org_id ,'not_globally_subscribable') = 0 OR exists (
						SELECT 1 from rhnChannelPermission per where per.channel_id = ct.id
						)
                            OR (rhn_user.check_role(ct.user_id, 'org_admin') = 1
                                OR rhn_user.check_role(ct.user_id, 'channel_admin') = 1);

--
-- 34. RHN_CHANNEL_RU_CK  (Constraint)
--
 ALTER TABLE RHNCHANNEL ADD CONSTRAINT RHN_CHANNEL_RU_CK CHECK (receiving_updates in ('Y', 'N'));

--
-- 35. RHN_CHANNEL_RU_NN  (Constraint)
--
-- ALTER TABLE RHNCHANNEL ADD CONSTRAINT RHN_CHANNEL_RU_NN CHECK ("RECEIVING_UPDATES" IS NOT NULL);

--
-- 36. RHN_CHANNEL_PRODUCT_NAME_CH_FK  (Constraint)
--
--ALTER TABLE RHNCHANNEL ADD CONSTRAINT RHN_CHANNEL_PRODUCT_NAME_CH_FK FOREIGN KEY (PRODUCT_NAME_ID) REFERENCES RHNPRODUCTNAME (ID);

--
-- 37. RHN_KS_CC_CID_FK  (Constraint)
--
ALTER TABLE RHNKICKSTARTCHILDCHANNEL DROP CONSTRAINT RHN_KS_CC_CID_FK;
ALTER TABLE RHNKICKSTARTCHILDCHANNEL ADD CONSTRAINT RHN_KS_CC_CID_FK FOREIGN KEY (CHANNEL_ID)
    REFERENCES RHNCHANNEL (ID)
    ON DELETE CASCADE;

--
-- 38. PRODUCT_NAME_CREATED_NN  (Constraint)
--
--ALTER TABLE RHNPRODUCTNAME ADD CONSTRAINT PRODUCT_NAME_CREATED_NN CHECK ("CREATED" IS NOT NULL);

--
-- 39. PRODUCT_NAME_MODIFIED_NN  (Constraint)
--
--ALTER TABLE RHNPRODUCTNAME ADD CONSTRAINT PRODUCT_NAME_MODIFIED_NN CHECK ("MODIFIED" IS NOT NULL);

--
-- 40. RHN_PRODUCTNAME_ID_NN  (Constraint)
--
--ALTER TABLE RHNPRODUCTNAME ADD CONSTRAINT RHN_PRODUCTNAME_ID_NN CHECK ("ID" IS NOT NULL);

--
-- 41. RHN_PRODUCTNAME_LBL_NN  (Constraint)
--
--ALTER TABLE RHNPRODUCTNAME ADD CONSTRAINT RHN_PRODUCTNAME_LBL_NN CHECK ("LABEL" IS NOT NULL);

--
-- 42. RHN_PRODUCTNAME_NAME_NN  (Constraint)
--
--ALTER TABLE RHNPRODUCTNAME ADD CONSTRAINT RHN_PRODUCTNAME_NAME_NN CHECK ("NAME" IS NOT NULL);

--
-- 43. RHN_PRODUCTNAME_ID_PK  (Constraint)
--
ALTER TABLE RHNPRODUCTNAME ADD CONSTRAINT RHN_PRODUCTNAME_ID_PK PRIMARY KEY (ID);

--
-- 36. RHN_CHANNEL_PRODUCT_NAME_CH_FK  (Constraint)
--
ALTER TABLE RHNCHANNEL ADD CONSTRAINT RHN_CHANNEL_PRODUCT_NAME_CH_FK FOREIGN KEY (PRODUCT_NAME_ID) REFERENCES RHNPRODUCTNAME (ID);

--
-- 44. RHN_RCM_CAID_NN  (Constraint)
--
--ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_CAID_NN CHECK ("CHANNEL_ARCH_ID" IS NOT NULL);

--
-- 45. RHN_RCM_CID_NN  (Constraint)
--
-- ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_CID_NN CHECK ("CHANNEL_ID" IS NOT NULL);

--
-- 46. RHN_RCM_DEFAULT_CK  (Constraint)
--
ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_DEFAULT_CK CHECK (is_default in ('Y', 'N'));

--
-- 47. RHN_RCM_DEFAULT_NN  (Constraint)
--
--ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_DEFAULT_NN CHECK ("IS_DEFAULT" IS NOT NULL);

--
-- 48. RHN_RCM_PRODCT_NN  (Constraint)
--
--ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_PRODCT_NN CHECK ("PRODUCT" IS NOT NULL);

--
-- 49. RHN_RCM_RELSE_NN  (Constraint)
--
--ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_RELSE_NN CHECK ("RELEASE" IS NOT NULL);

--
-- 50. RHN_RCM_VERSN_NN  (Constraint)
--
--ALTER TABLE RHNRELEASECHANNELMAP ADD CONSTRAINT RHN_RCM_VERSN_NN CHECK ("VERSION" IS NOT NULL);

--
-- 51. RHN_CHP_CREATED_NN  (Constraint)
--
--ALTER TABLE RHNCHANNELPARENT ADD CONSTRAINT RHN_CHP_CREATED_NN CHECK ("CREATED" IS NOT NULL);

--
-- 52. RHN_CHP_MODIFIED_NN  (Constraint)
--
--ALTER TABLE RHNCHANNELPARENT ADD CONSTRAINT RHN_CHP_MODIFIED_NN CHECK ("MODIFIED" IS NOT NULL);

--
-- 53. RHN_CP_CH_NN  (Constraint)
--
--ALTER TABLE RHNCHANNELPARENT ADD CONSTRAINT RHN_CP_CH_NN CHECK ("CHANNEL" IS NOT NULL);

--
-- 54. RHN_CP_PARENT_CH_NN  (Constraint)
--
--ALTER TABLE RHNCHANNELPARENT ADD CONSTRAINT RHN_CP_PARENT_CH_NN CHECK ("PARENT_CHANNEL" IS NOT NULL);

--
-- 55. RHN_CP_CH_FK  (Constraint)
--
ALTER TABLE RHNCHANNELPARENT ADD CONSTRAINT RHN_CP_CH_FK FOREIGN KEY (CHANNEL)
    REFERENCES RHNCHANNEL (ID)
    ON DELETE CASCADE;

--
-- 56. RHN_CP_PARENT_CH_FK  (Constraint)
--
ALTER TABLE RHNCHANNELPARENT ADD CONSTRAINT RHN_CP_PARENT_CH_FK FOREIGN KEY (PARENT_CHANNEL)
    REFERENCES RHNCHANNEL (ID);

--
-- 57. RHN_VIS_OBJS_SESS_FK  (Constraint)
--
ALTER TABLE RHNVISIBLEOBJECTS ADD CONSTRAINT RHN_VIS_OBJS_SESS_FK FOREIGN KEY (PXT_SESSION_ID)
    REFERENCES PXTSESSIONS (ID)
    ON DELETE CASCADE;

--
-- rhn_clientcapnam_name_unq
--
create unique index rhn_clientcapnam_name_unq on rhnClientCapabilityName(name);

--
-- rhn_kvt_label_unq
--
create unique index rhn_kvt_label_unq on rhnKickstartVirtualizationType(label);

--
-- rhnksdata, column postlog
--
alter table rhnksdata add "POSTLOG" CHAR(1) DEFAULT ('N') CONSTRAINT "RHN_KS_POST_LOG_NN" NOT NULL;
alter table rhnksdata add CONSTRAINT "RHN_KS_POST_LOG_CK" CHECK (postLog in ('Y','N'));

--
-- rhn_user_group_org_mapping
--
CREATE OR REPLACE TRIGGER
rhn_user_group_org_mapping
BEFORE INSERT OR UPDATE ON rhnUserGroupMembers
FOR EACH ROW
DECLARE
        same_org        NUMBER;
BEGIN
        same_org := 0;
        SELECT 1 INTO same_org
          FROM web_contact U, rhnUserGroup UG
         WHERE UG.org_id = U.org_id
           AND U.id = :new.user_id
           AND UG.id = :new.user_group_id;

        IF same_org = 0 THEN
          rhn_exception.raise_exception('ugm_different_orgs');
        END IF;
EXCEPTION
        WHEN NO_DATA_FOUND THEN
          rhn_exception.raise_exception('ugm_different_orgs');
END;
/
SHOW ERRORS

--
-- Update the trigger rhn_errata_package_mod_trig to mark errata modified

create or replace trigger
rhn_errata_package_mod_trig
before insert or update or delete on rhnErrataPackage
for each row
begin
	if inserting or updating then
		:new.modified := sysdate;
	end if;
	if deleting then
		update rhnErrata
		set rhnErrata.last_modified = sysdate
		where rhnErrata.id in ( :old.errata_id );
	end if;
end rhn_errata_package_mod_trig;
/
show errors

--
-- 1. RHN_CHANNEL  (Package Body)
--
ALTER PACKAGE RHN_CHANNEL COMPILE BODY;

--
-- 2. RHN_ENTITLEMENTS  (Package Body)
--
ALTER PACKAGE RHN_ENTITLEMENTS COMPILE BODY;

--
-- 3. RHN_ORG  (Package Body)
--
ALTER PACKAGE RHN_ORG COMPILE BODY;

--
-- 4. RHN_PACKAGE  (Package Body)
--
ALTER PACKAGE RHN_PACKAGE COMPILE BODY;

--
-- 5. RHN_SERVER  (Package Body)
--
ALTER PACKAGE RHN_SERVER COMPILE BODY;

--
-- 6. RHNCHANNEL  (Table)
--
-- Difference : Column Order (No action taken)

--
-- 7. RHN_CHANNEL_MOD_TRIG  (Trigger)
--
ALTER TRIGGER RHN_CHANNEL_MOD_TRIG COMPILE;

--
-- 8. RHN_PACKAGE  (Package)
--
ALTER PACKAGE RHN_PACKAGE COMPILE;

--
-- 9. RHN_CHANNEL_DEL_TRIG  (Trigger)
--
ALTER TRIGGER RHN_CHANNEL_DEL_TRIG COMPILE;

--
-- 10. DELETE_CHANNEL  (Procedure)
--
ALTER PROCEDURE DELETE_CHANNEL COMPILE;

--
-- 11. DELETE_SERVER_BULK  (Procedure)
--
ALTER PROCEDURE DELETE_SERVER_BULK COMPILE;

--
-- Table data changes
--
insert into rhnActionType values (49, 'proxy.deactivate', 'Deactivate Proxy
', 'N', 'N');
insert into rhnException
values (-20290,
        'not_enough_entitlements_in_base_org',
        'You do not have enough entitlements in the base org.');

insert into rhnException
values (-20291,
        'cannot_delete_base_org',
        'You cannot delete the base org.');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'ignoredisk', 'Y', 35, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'autopart', 'N', 36, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'cmdline', 'N', 37, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'firstboot', 'Y', 38, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'graphical', 'N', 39, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'iscsi', 'Y', 40, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'iscsiname', 'Y', 41, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'logging', 'Y', 42, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'monitor', 'Y', 43, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'multipath', 'Y', 44, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'poweroff', 'N', 45, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'halt', 'N', 46, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'services', 'Y', 47, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'shutdown', 'N', 48, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'user', 'Y', 49, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'vnc', 'Y', 50, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'zfcp', 'Y', 51, 'N');

insert into rhnKickstartCommandName (id, name, uses_arguments, sort_order,required)
values (rhn_kscommandname_id_seq.nextval, 'custom', 'Y', 52, 'N');

--
-- data for rhnUserGroupType
--
 insert into rhnUserGroupType (id, label, name) values (
        rhn_usergroup_type_seq.nextval,
        'satellite_admin',
        'Satellite Administrator'
);

--
-- data for rhnTimezone
--
insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'GB', 'Great Britain');

insert into rhnTimezone
  (id, olson_name, display_name)
values
  (rhn_timezone_id_seq.nextval,
   'GB-Eire', 'Ireland');

--
-- promote one organization to satellite_admin
--
insert into rhnUserGroup (
        id, name, description,
        max_members, group_type, org_id
        )
select rhn_user_group_id_seq.nextval,
        replace(rhnUserGroup.name, 'Organization', 'Satellite'),
        replace(rhnUserGroup.description, 'Organization', 'Satellite'),
        null,
        group_satellite_admin_type.id,
        first_org.id
from rhnUserGroup,
        rhnUserGroupType group_satellite_admin_type,
        rhnUserGroupType group_org_admin_type,
        (
                select min(id) id
                from web_customer
        ) first_org
where group_satellite_admin_type.label = 'satellite_admin'
        and group_org_admin_type.label = 'org_admin'
        and first_org.id = rhnUserGroup.org_id
        and rhnUserGroup.group_type = group_org_admin_type.id
        ;

--
-- promote all org_admins to satellite_admin
--
insert into rhnUserGroupMembers (
	user_id, user_group_id
	)
select user_id, group_satellite_admin.id
from rhnUserGroupMembers,
	rhnUserGroupType group_satellite_admin_type,
	rhnUserGroupType group_org_admin_type,
	rhnUserGroup group_satellite_admin,
	rhnUserGroup group_org_admin
where group_org_admin_type.label = 'org_admin'
	and group_org_admin_type.id = group_org_admin.group_type
	and rhnUserGroupMembers.user_group_id = group_org_admin.id
	and group_satellite_admin_type.label = 'satellite_admin'
	and group_satellite_admin_type.id = group_satellite_admin.group_type
	and group_org_admin.org_id = group_satellite_admin.org_id;

--
-- make sure organizations start at two
-- since create_first_org has a hardcoded one
--
select web_customer_id_seq.nextval from dual;

--
-- update password params to be password widgets
--
update rhn_command_parameter set field_widget_name = 'password'
   where param_name like '%pass%';

--
-- update local kickstarts to start with only the slash
--
update rhnKickstartCommand
set arguments = '--url ' || substr(arguments, instr(arguments, '/', 14))
where ks_command_name_id in ( select id from rhnKickstartCommandName where name = 'url' )
	and substr(arguments, 1, 13) = '--url http://'
	and instr(arguments, '/', 14) > 14
	and substr(arguments, instr(arguments, '/', 14), 15) = '/rhn/kickstart/' ;
