-------------------------------------------------------------------------------
-- Execute this script in a 3.6-199 schema to upgrade it to 3.6-214
-------------------------------------------------------------------------------

set echo on;
whenever sqlerror exit failure;

spool satellite-3.6-199-to-3.6-214.log;

variable evr_id number;
variable epoch varchar2(16);
variable version varchar2(64);
variable release varchar2(64);
variable message varchar2(80);

set define off;
set sqlblanklines on;

declare
   cursor evrs is
      select   e.id, e.epoch, e.version, e.release, e.evr
      from  rhnPackageEVR e,
         rhnVersionInfo rvi
      where rvi.label = 'schema'
         and rvi.name_id =
            lookup_package_name('rhn-satellite-schema')
         and rvi.evr_id = e.id;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','199')
         );
begin
   :evr_id := null;
   :message := 'XXX Invalid satellite schema version.';
   for evr in evrs loop
      :evr_id := evr.id;
      :epoch := evr.epoch;
      :version := evr.version;
      :release := evr.release;
      :message :=  '*** Schema version is currently ' ||
         evr.evr.as_vre_simple() ||
         ', and will NOT be upgraded';
      for vevr in valid_evrs loop
         :message :=  '*** Schema version is currently ' ||
            evr.evr.as_vre_simple() ||
            ', and will be upgraded';
      end loop;
      return;
   end loop;
end;
/

select :message from dual;

declare
   invalid_schema_version exception;
   cursor valid_evrs is
      select   1
      from  dual
      where :evr_id in (
         lookup_evr('','3.6','199')
         );
begin
   for vevr in valid_evrs loop
      return;
   end loop;
   raise invalid_schema_version;
-- exception
--    when invalid_schema_version
--    then
--       return;
end;
/
show errors

-- 
-- Drop Triggers
-- 1. RHN_KSTREEFILE_DEL_TRIG (Trigger)
-- 2. RHN_EFILEC_DEL_TRIG  (Trigger) 
--
DROP TRIGGER RHN_KSTREEFILE_DEL_TRIG;
DROP TRIGGER RHN_EFILEC_DEL_TRIG;

--
-- 3. RHN_EFILEC_MOD_TRIG  (Trigger) 
--
CREATE OR REPLACE TRIGGER RHN_EFILEC_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNERRATAFILECHANNEL
FOR EACH ROW
begin
   :new.modified := sysdate;
end rhn_efilec_mod_trig;
/
SHOW ERRORS;

--
-- 4. RHN_KSTREEFILE_MOD_TRIG  (Trigger) 
--
CREATE OR REPLACE TRIGGER RHN_KSTREEFILE_MOD_TRIG
BEFORE INSERT OR UPDATE ON RHNKSTREEFILE
FOR EACH ROW
begin
   :new.modified := sysdate;
   if :new.last_modified = :old.last_modified
   then
       :new.last_modified := sysdate;
        end if;
end rhn_kstreefile_mod_trig;
/
SHOW ERRORS;

--
-- update data for rhnFAQ
--
update rhnFAQ
set details = 
'Red Hat Network is a systems support and management environment for Red Hat Linux systems and networks. For more information, please see the Red Hat Network product information page:
http://www.redhat.com/software/rhn/products/

For individual systems and small networks, see http://www.redhat.com/software/rhn

For enterprise deployments, see http://www.redhat.com/software/rhen'
where id = 1;

commit;

----------------------------------------------
update rhnVersionInfo set evr_id = lookup_evr(null, '3.6', '214')
   where label = 'schema'
   and name_id = lookup_package_name('rhn-satellite-schema');
commit;

select   '*** Schema version is now ' || e.evr.as_vre_simple()
from  rhnPackageEVR e, rhnVersionInfo vi
where vi.evr_id = e.id
  and   vi.label = 'schema';

spool off;
exit;

